using System;
using System.Collections;

namespace BBN.DAML
{
	/// <summary>
	/// 
	/// </summary>
	public class ConcatenatedEnumerator : IEnumerator
	{

		IEnumerator e1;
		IEnumerator e2;
		IEnumerator curr;
		object m_defaultValue = null;
		bool m_defaultValueSeen = false;
		bool m_doDefault = false;

		public ConcatenatedEnumerator(IEnumerator e1, IEnumerator e2)
		{
			this.e1 = e1;
			this.e2 = e2;
			curr = e1;
		}

		public void SetDefaultValue( object defaultValue) 
		{
			m_defaultValue = defaultValue;
		}

		public bool HasDefaultValue() 
		{
			return m_defaultValue != null;
		}

		public void Reset() 
		{
			e1.Reset();
			e2.Reset();
			curr = e1;
			m_defaultValueSeen = false;
			m_doDefault = false;
		}

		public bool MoveNext()
		{ 
			m_doDefault = false;
			if (!curr.MoveNext()) 
			{
				if (curr == e1) 
				{
					curr = e2;
					return curr.MoveNext();
				}
				else if (HasDefaultValue() && !m_defaultValueSeen)
				{
					m_doDefault = true;
					return true;
				}
				else
				{
					return false;
				}
			}
			else 
			{
				return true;
			}
		}

		public object Current 
		{
			get 
			{
				if (m_doDefault)
				{ 
					m_defaultValueSeen = true;
					return m_defaultValue;
				} 
				else
				{
					// is this the default value?
					if (HasDefaultValue()  &&  m_defaultValue.Equals( curr.Current )) 
					{
						m_defaultValueSeen = true;
					}
					return curr.Current;
				}
			}
		}

	}
}
