using System;
using System.Collections;
using BBN.RDF;

namespace BBN.DAML
{
	/// <summary>
	/// 
	/// </summary>
	public class ConcatenatedNodeEnumerator: NodeEnumerator
{

		NodeEnumerator e1;
		NodeEnumerator e2;
		NodeEnumerator curr;
		RDFNode m_defaultValue = null;
		bool m_defaultValueSeen = false;
		bool m_doDefault = false;

		public ConcatenatedNodeEnumerator(NodeEnumerator e1, NodeEnumerator e2)
		{
			this.e1 = e1;
			this.e2 = e2;
			curr = e1;
		}

		public void SetDefaultValue( RDFNode defaultValue) 
		{
			m_defaultValue = defaultValue;
		}

		public bool HasDefaultValue() 
		{
			return m_defaultValue != null;
		}

		public void Reset() 
		{
			e1.Reset();
			e2.Reset();
			curr = e1;
			m_defaultValueSeen = false;
			m_doDefault = false;
		}

		public bool MoveNext()
		{ 
			m_doDefault = false;
			if (curr.MoveNext() == false) 
			{
				if (curr == e1) 
				{
					curr = e2;
					return curr.MoveNext();
				}
				else if (HasDefaultValue() && !m_defaultValueSeen)
				{
					m_doDefault = true;
					return true;
				}
			{ 
				return false;
			}
			}
			else 
			{
				return true;
			}
		}

		public RDFNode Current 
		{
			get 
			{
				if (m_doDefault)
				{ 
					m_defaultValueSeen = true;
					return m_defaultValue;
				} 
				else
				{
					// is this the default value?
					if (HasDefaultValue()  &&  m_defaultValue.Equals( curr.Current )) 
					{
						m_defaultValueSeen = true;
					}
					return curr.Current;
				}
			}
		}

		object IEnumerator.Current
		{
			get
			{
				return (object)this.Current;
			}
		}


		/// <summary>
		/// This basically does nothing.
		/// </summary>
		public void Close()
		{}


	}
}
