using System;
using System.Collections;
using BBN.DAML.Vocabulary;
using BBN.RDF.Vocabulary;
using BBN.RDF;

namespace BBN.DAML
{
	/// <summary>
	/// 
	/// </summary>
	
	public class DAMLClassImpl : DAMLCommonImpl, DAMLClass
	{

		// Instance variables
		//////////////////////////////////

		/** Property accessor for onProperty */
		private PropertyAccessor m_propSubClassOf = null;

		/** Property accessor for disjointWith */
		private PropertyAccessor m_propDisjointWith = null;

		/** Property accessor for disjointUnionOf */
		private PropertyAccessor m_propDisjointUnionOf = null;

		/** Property accessor for sameClassAs */
		private PropertyAccessor m_propSameClassAs = null;

		/** Property accessor for oneOf */
		private PropertyAccessor m_propOneOf = null;

		/** Property accessor for unionOf */
		private PropertyAccessor m_propUnionOf = null;

		/** Property accessor for intersectionOf */
		private PropertyAccessor m_propIntersectionOf = null;

		/** Property accessor for complementOf */
		private PropertyAccessor m_propComplementOf = null;

		// Constructors
		//////////////////////////////////

		/// <summary>
		/// Constructor, takes the name and namespace for this class, and the underlying
		/// model it will be attached to.
		/// </summary>
		/// <param name="ns">The namespace the class inhabits, or null</param>
		/// <param name="name">he name of the class</param>
		/// <param name="model">Reference to the DAML model that will contain statements about this DAML class.</param>
		/// <param name="vocabulary">Reference to the DAML vocabulary used by this class.</param>
		public DAMLClassImpl( string ns, string name, DAMLModel model, DAMLVocabulary vocabulary ) : base( ns, name, model, vocabulary )
		{
			SetRDFType( RDFS.Class );
		}


		/// <summary>
		/// Constructor, takes URI for this class, and the underlying
		/// model it will be attached to.
		/// </summary>
		/// <param name="uri">The URI of the class</param>
		/// <param name="store">Reference to the DAML store that will contain statements about this DAML class.</param>
		/// <param name="vocabulary">Reference to the DAML vocabulary used by this class.</param>
		public DAMLClassImpl( string uri, DAMLModel store, DAMLVocabulary vocabulary ) : base( uri, store, vocabulary )
		{
			SetRDFType( RDFS.Class );
		}




		// External signature methods
		//////////////////////////////////


		/**
		 * Property accessor for the 'subClassOf' property of a class. This
		 * denotes a class-expression that is a super-class of this class.
		 *
		 * @return Property accessor for 'onProperty'.
		 */
		public PropertyAccessor Prop_SubClassOf
		{
			get 
			{
				if (m_propSubClassOf == null)
				{
					m_propSubClassOf = new PropertyAccessorImpl( GetVocabulary().subClassOf, this );
				}
				return m_propSubClassOf;
			}
		}


		/**
		 * Property accessor for the 'disjointWith' property of a class. This
		 * denotes a class-expression with which this class has no instances in common.
		 *
		 * @return Property accessor for 'disjointWith'.
		 */
		public PropertyAccessor Prop_DisjointWith
		{
			get 
			{
				if (m_propDisjointWith == null) 
				{
					m_propDisjointWith = new PropertyAccessorImpl( GetVocabulary().disjointWith, this );
				}

				return m_propDisjointWith;
			}
		}


		/**
		 * Property accessor for the 'disjointUnionOf' property of a class. This
		 * denotes a list of class expressions that are each pair-wise disjoint, and whose
		 * union describes this class.
		 *
		 * @return Property accessor for 'disjointUnionOf'.
		 */
		public PropertyAccessor Prop_DisjointUnionOf
		{
			get 
			{
				if (m_propDisjointUnionOf == null) 
				{
					m_propDisjointUnionOf = new PropertyAccessorImpl( GetVocabulary().disjointUnionOf, this );
				}

				return m_propDisjointUnionOf;
			}
		}


		/**
		 * Property accessor for the 'sameClassAs' property of a class. This
		 * denotes a class-expression whose instances are the same those of this class.
		 *
		 * @return Property accessor for 'sameClassAs'.
		 */
		public PropertyAccessor Prop_SameClassAs
		{
			get 
			{
				if (m_propSameClassAs == null) 
				{
					m_propSameClassAs = new PropertyAccessorImpl( GetVocabulary().sameClassAs, this );
				}

				return m_propSameClassAs;
			}
		}


		/**
		 * Property accessor for the property 'unionOf', which is one element of the range
		 * of boolean expressions over classes permitted by DAML.
		 *
		 * @return property accessor for 'unionOf'.
		 */
		public PropertyAccessor Prop_UnionOf
		{
			get 
			{
				if (m_propUnionOf == null) 
				{
					m_propUnionOf = new PropertyAccessorImpl( GetVocabulary().unionOf, this );
				}

				return m_propUnionOf;
			}
		}


		/**
		 * Property accessor for the property 'intersectionOf', which is one element of the range
		 * of boolean expressions over classes permitted by DAML.
		 *
		 * @return property accessor for 'intersectionOf'.
		 */
		public PropertyAccessor Prop_IntersectionOf
		{
			get 
			{
				if (m_propIntersectionOf == null) 
				{
					m_propIntersectionOf = new PropertyAccessorImpl( GetVocabulary().intersectionOf, this );
				}

				return m_propIntersectionOf;
			}
		}


		/**
		 * Property accessor for the property 'compelementOf', which is one element of the range
		 * of boolean expressions over classes permitted by DAML.
		 *
		 * @return property accessor for 'complementOf'.
		 */
		public PropertyAccessor Prop_ComplementOf
		{
			get 
			{
				if (m_propComplementOf == null) 
				{
					m_propComplementOf = new PropertyAccessorImpl( GetVocabulary().complementOf, this );
				}

				return m_propComplementOf;
			}
		}


		/**
		 * Property accessor for the 'oneOf' property, which defines a class expression
		 * denoting that the class is exactly one of the given list of class expressions.
		 *
		 * @return property accessor for 'oneOf'
		 */
		public PropertyAccessor Prop_OneOf
		{
			get 
			{
				if (m_propOneOf == null) 
				{
					m_propOneOf = new PropertyAccessorImpl( GetVocabulary().oneOf, this );
				}

				return m_propOneOf;
			}
		}


		/**
		 * Answer true if this class expression is an enumeration (i.e. has a property
		 * 'oneOf' with a list of values).  This is not an exclusive property, a class
		 * expression can be an enumeration at the same time as one of the other kinds
		 * of class expression, though the conjunction of these may produce the Nothing
		 * class.
		 *
		 * @return true if this class expression is an enumeration.
		 */
		public virtual bool IsEnumeration
		{
			get 
			{
				try 
				{
					return HasProperty( GetVocabulary().oneOf );
				}
				catch (System.Exception e) 
				{
					Console.Error.WriteLine( "RDF exception " + e, e );
					throw new System.Exception( "RDF Exception " + e );
				}
			}
		}


		/**
		 * Answer true if this class expression is an named class (i.e. is not an anonymous
		 * class expression).  This is not an exclusive property, a class
		 * expression can be named at the same time as one of the other kinds
		 * of class expression, though the conjunction of these may produce the Nothing
		 * class.
		 *
		 * @return true if this class expression is a named class.
		 */
		public virtual bool IsNamedClass
		{
			get 
			{
				return !IsAnon();
			}
		}


		/**
		 * Answer true if this class expression is an property restriction (i.e. is a
		 * Restriction value).  This is not an exclusive property, a class
		 * expression can be a property restriction at the same time as one of the other kinds
		 * of class expression, though the conjunction of these may produce the Nothing
		 * class.
		 *
		 * @return necessarily false, since restrictions are represented by a different Java class (see
		 *         {@link com.hp.hpl.jena.daml.DAMLRestriction}).
		 */
		public virtual bool IsRestriction
		{
			get 
			{
				return false;
			}
		}


		/**
		 * Answer true if this class expression is an boolean intersection of a list
		 * of class expressions.  This is not an exclusive property, a class
		 * expression can be an intersection at the same time as one of the other kinds
		 * of class expression, though the conjunction of these may produce the Nothing
		 * class.
		 *
		 * @return true if this class expression is an intersection.
		 */
		public virtual bool IsIntersection
		{
			get 
			{
				try 
				{
					return HasProperty( GetVocabulary().intersectionOf );
				}
				catch (System.Exception e) 
				{
					Console.Error.WriteLine( "RDF exception " + e, e );
					throw new System.Exception( "RDF Exception " + e );
				}
			}
		}


		/**
		 * Answer true if this class expression is an boolean union of a list
		 * of class expressions.  This is not an exclusive property, a class
		 * expression can be an union at the same time as one of the other kinds
		 * of class expression, though the conjunction of these may produce the Nothing
		 * class.
		 *
		 * @return true if this class expression is a union.
		 */
		public virtual bool IsUnion
		{
			get 
			{
				try 
				{
					return HasProperty( GetVocabulary().unionOf );
				}
				catch (System.Exception e) 
				{
					Console.Error.WriteLine( "RDF exception " + e, e );
					throw new System.Exception( "RDF Exception " + e );
				}
			}
		}


		/**
		 * Answer true if this class expression is a disjoint union of a list
		 * of class expressions.  This is not an exclusive property, a class
		 * expression can be a disjoint union at the same time as one of the other kinds
		 * of class expression, though the conjunction of these may produce the Nothing
		 * class.
		 *
		 * @return true if this class expression is a disjoint union.
		 */
		public virtual bool IsDisjointUnion
		{
			get 
			{
				try 
				{
					return HasProperty( GetVocabulary().disjointUnionOf );
				}
				catch (System.Exception e) 
				{
					Console.Error.WriteLine( "RDF exception " + e, e );
					throw new System.Exception( "RDF Exception " + e );
				}
			}
		}


		/**
		 * Answer true if this class expression is an boolean complement of a list
		 * of class expressions.  This is not an exclusive property, a class
		 * expression can be an complement at the same time as one of the other kinds
		 * of class expression, though the conjunction of these may produce the Nothing
		 * class.
		 *
		 * @return true if this class expression is a complement.
		 */
		public virtual bool IsComplement
		{
			get 
			{
				try 
				{
					return HasProperty( GetVocabulary().complementOf );
				}
				catch (System.Exception e) 
				{
					Console.Error.WriteLine( "RDF exception " + e, e );
					throw new System.Exception( "RDF Exception " + e );
				}
			}
		}


		/**
		 * Answer an iterator over the DAML classes (or, strictly, class expressions)
		 * that mention this class as one of its super-classes.   Will generate the
		 * closure of the iteration over the sub-class relationship.
		 *
		 * @return an iterator over this class's sub-classes. The members of the
		 *         iteration will be DAMLClass objects.
		 */
		public IEnumerator GetSubClasses() 
		{
			return GetSubClasses( true );
		}


		/**
		 * Answer an iterator over the DAML classes (or, strictly, class expressions)
		 * that mention this class as one of its super-classes.
		 *
		 * @param closed If true, close the iteration over the sub-class relation: i.e.
		 *               return the sub-classes of the sub-classes, etc.
		 * @return an iterator over this class's sub-classes. The members of the
		 *         iteration will be DAMLClass objects.
		 */
		public IEnumerator GetSubClasses( bool closed ) 
		{
			return new PropertyEnumerator( this, null, GetVocabulary().subClassOf, closed, false, true );
		}



		/**
		* Answer an iterator over the DAML classes (or, strictly, class expressions)
		* that mention this class as one of its sub-classes.  Will generate the
		* closure of the iteration over the super-class relationship.
		*
		* @return an iterator over this class's super-classes. The members of the
		*         iteration will be DAMLClass objects.
		*/
		public IEnumerator GetSuperClasses() 
		{
			return GetSuperClasses( true );
		}


		/**
		 * Answer an iterator over the DAML classes (or, strictly, class expressions)
		 * that mention this class as one of its sub-classes.
		 *
		 * @param closed If true, close the iteration over the super-class relation: i.e.
		 *               return the super-classes of the super-classes, etc.
		 * @return an iterator over this class's sub-classes. The members of the
		 *         iteration will be DAMLClass objects.
		 */
		public IEnumerator GetSuperClasses( bool closed ) 
		{
			IEnumerator i = new PropertyEnumerator( this, GetVocabulary().subClassOf, null, closed, false, true );

			// ensure that the types always include Thing in the closure
			if (closed) 
			{
				((PropertyEnumerator) i).SetDefaultValue( GetVocabulary().Thing );
			}

			return i;
		}

		
		/**
		 * Answer a key that can be used to index collections of this DAML class for
		 * easy access by iterators.  Package access only.
		 *
		 * @return a key object.
		 */
		public override Object GetKey() 
		{
			return DAML_OIL._Class.GetURI();
		}

		/**


	/**
	 * Answer an iterator over all of the DAML classes that are equivalent to this
	 * value under the <code>daml:sameClassAs</code> relation.  Note: only considers
	 * <code>daml:sameClassAs</code>, for general equivalence, see
	 * {@link #GetEquivalentValues}.
	 *
	 * @return an iterator ranging over every equivalent DAML class - each value of
	 *         the iteration will be a DAMLClass object.
	 */
		public IEnumerator GetSameClasses() 
		{
			return new PropertyEnumerator( this, GetVocabulary().sameClassAs, GetVocabulary().sameClassAs, true, true, true );
		}



		/**
		 * Answer an iterator over all of the DAML objects that are equivalent to this
		 * class, which will be the union of <code>daml:equivalentTo</code> and
		 * <code>daml:sameClassAs</code>.
		 *
		 * @return an iterator ranging over every equivalent DAML class - each value of
		 *         the iteration should be a DAMLClass object.
		 */
		public override IEnumerator GetEquivalentValues() 
		{
			ConcatenatedEnumerator i = new ConcatenatedEnumerator(
				// first the iterator over the equivalentTo values
				base.GetEquivalentValues(),
				// followed by the sameClassAs values
				new PropertyEnumerator( this, GetVocabulary().sameClassAs, GetVocabulary().sameClassAs, true, false, false ) );

			// ensure that the iteration includes self
			i.SetDefaultValue( this );

			return i;
		}


		/**
		 * Answer true if the given class is a sub-class of this class, using information
		 * from the <code>rdf:subClassOf</code> or <code>daml:subClassOf</code> relation.
		 *
		 * @param cls A Resource representing a DAML Class
		 * @return True if this class is a super-class of the given class.
		 */
		public bool HasSubClass( Resource cls ) 
		{
			for (IEnumerator i = GetSubClasses();  i.MoveNext();  ) 
			{
				// note that we don't need to test for class equivalence here - that is taken care
				// of by the sub-class iterator
				if (i.Current.Equals( cls )) 
				{
					return true;
				}
			}

			return false;
		}


		/**
		 * Answer true if the given class is a super-class of this class, using information
		 * from the <code>rdf:subClassOf</code> or <code>daml:subClassOf</code> relation.
		 *
		 * @param cls A resource representing a DAML Class.
		 * @return True if this class is a sub-class of the given class.
		 */
		public bool HasSuperClass( Resource cls ) 
		{
			for (IEnumerator i = GetSuperClasses();  i.MoveNext();  ) 
			{
				// note that we don't need to test for class equivalence here - that is taken care
				// of by the sub-class iterator
				if (i.Current.Equals( cls )) 
				{
					return true;
				}
			}

			return false;
		}


		/**
		 * Answer an iterator over the instances of this class that currently exist
		 * in the model.
		 *
		 * @return An iterator over those instances that have this class as one of
		 *         the classes to which they belong
		 * @see com.hp.hpl.jena.daml.DAMLCommon#GetRDFTypes
		 */
		public IEnumerator GetInstances() 
		{
			return new PropertyEnumerator( new ConcatenatedEnumerator( GetSelfEnumerator(), GetSubClasses() ),
				null, BBN.RDF.Vocabulary.RDF.type, false, false, true );
		}


		/**
		 * Answer an iteration of the properties that may be used for
		 * instances of this class: i.e. the properties that have this class,
		 * or optionally one of its super-classes, as domain.  <b>Note:</b>
		 * does not include the sub-properties of the defined properties as
			 * part of the iteration.
		 *
		 * @return An iteration of the properties that have this class as domain
		 */
		public IEnumerator GetDefinedProperties() 
		{
			return GetDefinedProperties( true );
		}


		/**
		 * Answer an iteration of the properties that may be used for
		 * instances of this class: i.e. the properties that have this class,
		 * or optionally one of its super-classes, as domain.
		 *
		 * @param closed If true, close the iteration over the super-classes
		 *               of this class.
		 * @return An iteration of the properties that have this class as domain
		 */
		public IEnumerator GetDefinedProperties( bool closed ) 
		{
			// select either this class or this class and all of its superclasses
			IEnumerator roots = closed ? new ConcatenatedEnumerator( GetSelfEnumerator(), GetSuperClasses( true ) ) :
				GetSelfEnumerator();

			// now look for properties that mention the root class(es) as domain
			return new PropertyEnumerator( roots, null, GetVocabulary().domain, false, false, true );
		}



	}
	
}
