using System;
using BBN.DAML.Vocabulary;

namespace BBN.DAML
{
	/// <summary>
	/// Encapsulates a DAML dataype, that represents values from a concrete domain by
	/// encoding their type using XML schema.
	/// </summary>
	public class DAMLDatatypeImpl : DAMLCommonImpl, DAMLDatatype
	{

    // Constructors
    //////////////////////////////////

    /**
     * Constructor, takes the name and namespace for this datatype, and the underlying
     * model it will be attached to.
     *
     * @param namespace The namespace the datatype inhabits, or null
     * @param name The name of the datatype
     * @param model Reference to the DAML model that will contain statements about this DAML datatype.
     * @param vocabulary Reference to the DAML vocabulary used by this datatype.
     */
    public DAMLDatatypeImpl( string ns, string name, DAMLModel model, DAMLVocabulary vocabulary ) 
        : base( ns, name, model, vocabulary )
    {
        SetRDFType( GetVocabulary().Datatype );
    }


    /**
     * Constructor, takes URI for this datatype, and the underlying
     * model it will be attached to.
     *
     * @param uri The URI of the datatype
     * @param store Reference to the DAML store that will contain statements about this DAML datatype.
     * @param vocabulary Reference to the DAML vocabulary used by this datatype.
     */
    public DAMLDatatypeImpl( string uri, DAMLModel store, DAMLVocabulary vocabulary ) 
        : base( uri, store, vocabulary )
    {
        SetRDFType( GetVocabulary().Datatype );
    }



    // External signature methods
    //////////////////////////////////

    /**
     * Answer the translator that can map between Java values and the serialised
     * form that represents the value in the RDF graph.
     *
     * @return a datatype translator.
     */
    public DatatypeTranslator GetTranslator() {
        return GetDAMLModel().GetDatatypeRegistry().GetTranslator( GetURI() );
    }


    /**
     * Answer a key that can be used to index collections of this DAML value for
     * easy access by iterators.  Package access only.
     *
     * @return a key object.
     */
    public override Object GetKey() {
        return GetVocabulary().Datatype;
    }


	}
}
