using System;
using BBN.DAML.Vocabulary;
using BBN.RDF;

namespace BBN.DAML
{
	/// <summary>
	/// 
	/// </summary>
	public class DAMLDatatypePropertyImpl : DAMLPropertyImpl, DAMLDatatypeProperty
	{
		// Instance variables
		//////////////////////////////////

		/** Property accessor for range */
		private LiteralAccessor m_propDatatypeRange = null;



		// Constructors
		//////////////////////////////////

		/**
		 * Constructor, takes the name and namespace for this property, and the underlying
		 * model it will be attached to.
		 *
		 * @param namespace The namespace the property inhabits, or null
		 * @param name The name of the property
		 * @param store The RDF store that contains the RDF statements defining the properties of the property
		 * @param vocabulary Reference to the DAML vocabulary used by this property.
		 */
		public DAMLDatatypePropertyImpl( String ns, String name, DAMLModel store, DAMLVocabulary vocabulary ) 
			:        base( ns, name, store, vocabulary )
		{
			SetRDFType( GetVocabulary().DatatypeProperty );
		}


		/**
		 * Constructor, takes the URI for this property, and the underlying
		 * model it will be attached to.
		 *
		 * @param uri The URI of the property
		 * @param store The RDF store that contains the RDF statements defining the properties of the property
		 * @param vocabulary Reference to the DAML vocabulary used by this property.
		 */
		public DAMLDatatypePropertyImpl( String uri, DAMLModel store, DAMLVocabulary vocabulary ) 
			: base( uri, store, vocabulary )
		{
			SetRDFType( GetVocabulary().DatatypeProperty );
		}



		// External signature methods
		//////////////////////////////////


		/**
		 * Literal accessor for the 'range' property of a datatype property.
		 * This denotes the XSchema datatype expression that denotes the concrete
		 * type that is the range of the property.  Note that Java does not allow
		 * us to override DAMLProperty.prop_range() with a different return type, so
		 * we have to introduce a new name for the range of a datatype property, that
		 * is an alias for prop_range.
		 *
		 * @return Literal accessor for 'range'.
		 */
		/* This was removed in Jena-1.4.0
		public LiteralAccessor Prop_DatatypeRange 
		{
			get
			{
				if (m_propDatatypeRange == null) 
				{
					m_propDatatypeRange = new LiteralAccessorImpl( GetVocabulary().range, this );
				}

				return m_propDatatypeRange;
			} 
		}
		*/


		/**
		 * Answer a key that can be used to index collections of this DAML property for
		 * easy access by iterators.  Package access only.
		 *
		 * @return a key object.
		 */
		public override Object GetKey() 
		{
			return DAML_OIL._Property.GetURI();
		}


		/**
		 * This is a Jena internal method.  I need to override the one in ResourceImpl
		 * to prevent a ClassCastException in some circumstances.
		 *
		 * @param m The model to port to
		 * @return The ported resource
		 * @throws RDFException if an RDF processing error ocurrs.
		 */
		public override Resource Port( Model m )
		{
			if ( GetModel() == m ) 
			{
				return this;
			}
			else 
			{
				if (m is DAMLModel) 
				{
					// the model is a DAML model, so we create a new DAML property
					DAMLDatatypePropertyImpl p = new DAMLDatatypePropertyImpl( GetNameSpace(), GetLocalName(), ((DAMLModel) m), GetVocabulary() );
					p.id = id;
					return p;
				}
				else 
				{
					// the model is not a DAML model, so return a generic property
					return m.GetProperty( GetURI() );
				}
			}
		}


	}
}
