using System;
using System.Collections;
using BBN.DAML.Vocabulary;
using BBN.RDF;

namespace BBN.DAML
{
	/// <summary>
	/// 
	/// </summary>
	public class DAMLInstanceImpl : DAMLCommonImpl, DAMLInstance
	{

		// Instance variables
		//////////////////////////////////

		/** Property accessor for sameIndividualAs */
		protected PropertyAccessor m_propsameIndividualAs = null;



		// Constructors
		//////////////////////////////////

		/**
		 * Constructor, takes the name and namespace for this instance, and the underlying
		 * model it will be attached to.  Note that it is assumed that the RDF store
		 * will contain a statement of the class to which this instance belongs.
		 *
		 * @param namespace The namespace the instance inhabits, or null
		 * @param name The name of the instance
		 * @param store The RDF store that contains the RDF statements defining the properties of the instance
		 * @param vocabulary Reference to the DAML vocabulary used by this instance.
		 */
		public DAMLInstanceImpl( string ns, string name, DAMLModel store, DAMLVocabulary vocabulary ) 
			: base( ns, name, store, vocabulary )
		{}


		/**
		 * Constructor, takes the URI for this instance, and the underlying
		 * model it will be attached to.  Note that it is assumed that the RDF store
		 * will contain a statement of the class to which this instance belongs.
		 *
		 * @param uri The URI of the instance
		 * @param store The RDF store that contains the RDF statements defining the properties of the instance
		 * @param vocabulary Reference to the DAML vocabulary used by this instance.
		 */
		public DAMLInstanceImpl( string uri, DAMLModel store, DAMLVocabulary vocabulary )
			: base( uri, store, vocabulary )
		{}




		// External signature methods
		//////////////////////////////////


		/**
		 * Answer a key that can be used to index collections of this DAML instance for
		 * easy access by iterators.  Package access only.
		 *
		 * @return a key object.
		 */
		public override Object GetKey() 
		{
			return DAML_OIL._Thing.GetURI();
		}


		/**
		 * Property accessor for <code>daml:sameIndividualAs</code> property on a DAML instance.
		 *
		 * @return a property accessor
		 */
		public PropertyAccessor Prop_SameIndividualAs 
		{
			get
			{
				if (m_propsameIndividualAs == null) 
				{
					m_propsameIndividualAs = new PropertyAccessorImpl( GetVocabulary().sameIndividualAs, this );
				}

				return m_propsameIndividualAs;
			}
		}


		/**
		 * Return an iterator over all of the instances that are the same as this one,
		 * by generating the transitive closure over the <code>daml:samePropertyAs</code>
		 * property.
		 *
		 * @return an iterator whose values will all be DAMLInstance objects
		 */
		public IEnumerator GetSameInstances() 
		{
			return new PropertyEnumerator( this, GetVocabulary().sameIndividualAs,
				GetVocabulary().sameIndividualAs, true, true, true );
		}



		/**
		 * Answer an iterator over all of the DAML objects that are equivalent to this
		 * instance, which will be the union of <code>daml:equivalentTo</code> and
		 * <code>daml:sameIndividualAs</code>.
		 *
		 * @return an iterator ranging over every equivalent DAML instance - each value of
		 *         the iteration should be a DAMLInstance object.
		 */
		public override IEnumerator GetEquivalentValues() 
		{
			ConcatenatedEnumerator i = new ConcatenatedEnumerator(
				// first the iterator over the equivalentTo values
				base.GetEquivalentValues(),
				// followed by the sameClassAs values
				new PropertyEnumerator( this, GetVocabulary().sameIndividualAs, GetVocabulary().sameIndividualAs, true, false, false ) );

			// ensure that the iteration includes self
			i.SetDefaultValue( this );

			return i;
		}


		/**
		 * Answer a property accessor for a user defined property.  TODO: think of
		 * a better name for this method!
		 *
		 * @param property An RDF or DAML property
		 * @return a property accessor, that simplifies some of the basic operations
		 *         of a given property on a given object
		 */
		public PropertyAccessor AccessProperty( Property property ) 
		{
			return new PropertyAccessorImpl( property, this );
		}


		// Internal implementation methods
		//////////////////////////////////

		/**
		 * Answer a value that will be a default type to include in an iteration of
		 * the value's rdf types.  Typically there is no default (null), but for an
		 * instance we want to ensure that the default type is daml:Thing.
		 *
		 * @return The default type: <code>daml:Thing</code>
		 */
		protected override Resource GetDefaultType() 
		{
			return GetVocabulary().Thing;
		}


	}
}
