using System;
using System.Collections;
using BBN.RDF.Vocabulary;
using BBN.RDF.Mem;
using BBN.RDF;
using BBN.RDF.Common;
using BBN.DAML.Vocabulary;

namespace BBN.DAML
{
	/// <summary>
	/// This is going to be ugly to implement!!!
	/// </summary>
 	//  TBD - Make the underlying model implementation independent
	public class DAMLModelImpl : ModelMem, DAMLModel
	{

		// Static variables
		//////////////////////////////////

		/** Encodes which Java class to instantiate for a given DAML class */

		protected static Object[,] DAML_CLASS_TABLE = new Object[,] {
																		// DAML class instance                   Corresponding C# class
		{ DAML_OIL._Class,                        "BBN.DAML.DAMLClassImpl" },
		{ DAML_OIL_2000_12._Class,                "BBN.DAML.DAMLClassImpl" },
		{ RDFS.Class,                            "BBN.DAML.DAMLClassImpl" },

		{ DAML_OIL_2000_12._Disjoint,             "BBN.DAML.DAMLDisjointImpl" },

		{ DAML_OIL._Restriction,                  "BBN.DAML.DAMLRestrictionImpl" },
		{ DAML_OIL_2000_12._Restriction,          "BBN.DAML.DAMLRestrictionImpl" },

		{ DAML_OIL._List,                         "BBN.DAML.DAMLListImpl" },
		{ DAML_OIL_2000_12._List,                 "BBN.DAML.DAMLListImpl" },

		{ DAML_OIL._Ontology,                     "BBN.DAML.DAMLOntologyImpl" },
		{ DAML_OIL_2000_12._Ontology,             "BBN.DAML.DAMLOntologyImpl" },

		{ DAML_OIL._Property,                     "BBN.DAML.DAMLPropertyImpl" },
		{ DAML_OIL_2000_12._Property,             "BBN.DAML.DAMLPropertyImpl" },
		{ BBN.RDF.Vocabulary.RDF.Property,        "BBN.DAML.DAMLPropertyImpl" },

		{ DAML_OIL._DatatypeProperty,             "BBN.DAML.DAMLDatatypePropertyImpl" },
		{ DAML_OIL._ObjectProperty,               "BBN.DAML.DAMLObjectPropertyImpl" },

		{ DAML_OIL_2000_12._UniqueProperty,       "BBN.DAML.DAMLPropertyImpl" },
		{ DAML_OIL_2000_12._TransitiveProperty,   "BBN.DAML.DAMLPropertyImpl" },
		{ DAML_OIL_2000_12._UnambiguousProperty,  "BBN.DAML.DAMLPropertyImpl" },

		{ DAML_OIL._UniqueProperty,               "BBN.DAML.DAMLPropertyImpl" },
		{ DAML_OIL._TransitiveProperty,           "BBN.DAML.DAMLObjectPropertyImpl" },
		{ DAML_OIL._UnambiguousProperty,          "BBN.DAML.DAMLObjectPropertyImpl" }
																	};


		// Instance variables
		//////////////////////////////////

		/** The loader that will load DAML source documents for this store */
		private DAMLLoader m_loader;

		/*
		 * The following looks complex, because it is rather.  Here's a summary: references to all of the
		 * DAML ontology wrapper objects we create are stored in m_stores.  Each entry in the array is
		 * an ArrayList corresponding to the objects of type, say, DAMLClass.  This allows us to iterate
		 * over DAML classes as a set. m_nameIndex and m_namespaceIndex map URI components to values
		 * (1 to many), and m_uriIndex maps URI to value (1 to 1).
		 */

		/** The set of indexes to different daml values */
		protected Hashtable m_indexes = new Hashtable();

		/** An index of values by type and name only (i.e. collecting namespace entries together) */
		//protected OneToManyMap m_nameIndex = new OneToManyMap();

		/** An index of classes by namespace and type only (i.e. collecting named entries together) */
		//protected OneToManyMap m_namespaceIndex = new OneToManyMap();

		/** A registry for mapping XML data to Java types */
		protected XMLDatatypeRegistry m_xmlDatatypeRegistry = new XMLDatatypeRegistry();

		/** Flag to control whether we use equivalence classes or not. Default true */
		protected bool m_useEquivalence = true;

		// Constructors
		//////////////////////////////////

		/**
		 * Constructor, initialises internal data structures.
		 */
		public DAMLModelImpl() 
		{
			m_loader = new DAMLLoader( this );
			// create well-known values
			InitStore();
		}




		// External signature methods
		//////////////////////////////////

    /**
     * Create an (optionally anonymous) Ontology (big-'O') element,
     * which holds meta-information for the ontology (small-'o').
     * <b>N.B.</b> This does not create a new
     * ontology, it simply makes an entry in the current model.
     *
     *
     * @param uri The URI for the new Ontology, or null to create an anonymous
     *            Ontology. Ideally provide the URL in which the Ontology is
     *            stored.
     *            Conventionally, in the RDF/XML serialization
     *            <pre>
     *             &lt;daml:Ontology rdf:about=""&gt;
     *            </pre>
     *            The empty URIref in the above RDF/XML is known as a
     *            <Q>same document reference</Q> and expands to the
     *            URL of the current file.
     *
     * @return A new DAMLOntology object, which is created by adding the
     *         appropriate statements to the RDF model.
     */
			public DAMLOntology CreateDAMLOntology( string uri ) {
				return (DAMLOntology) Index( new DAMLOntologyImpl( uri, this, VocabularyManager.GetDefaultVocabulary() ) );
			}



		/**
		 * Create an (optionally anonymous) instance of the given class.
		 *
		 * @param damlClass The class of the newly created DAMLInstance
		 * @param uri The URI for the new instance, or null to create an anonymous instance.
		 * @return A new DAMLInstance object, which is created by adding the
		 *         appropriate statements to the RDF model.
		 */
		public DAMLInstance CreateDAMLInstance( DAMLClass damlClass, string uri ) 
		{
			DAMLInstance instance = new DAMLInstanceImpl( uri, this, VocabularyManager.GetDefaultVocabulary() );
			instance.SetRDFType( damlClass );
			return (DAMLInstance) Index( instance );
		}


		/**
		 * Create an (optionally anonymous) DAML class.
		 *
		 * @param uri The URI for the new class, or null to create an anonymous class.
		 * @return A new DAMLClass object, which is created by adding the
		 *         appropriate statements to the RDF model.
		 */
		public DAMLClass CreateDAMLClass( string uri ) 
		{
			return (DAMLClass) Index( new DAMLClassImpl( uri, this, VocabularyManager.GetDefaultVocabulary() ) );
		}


		/**
		 * Create an (optionally anonymous) DAML property. Note that it is recommended
		 * to use one of the more specific property classes from the new DAML release:
		 * see {@link #createDAMLObjectProperty} or {@link #createDAMLDatatypeProperty}.
		 *
		 * @param uri The URI for the new property, or null to create an anonymous property.
		 * @return A new DAMLProperty object, which is created by adding the
		 *         appropriate statements to the RDF model.
		 */
		public DAMLProperty CreateDAMLProperty( string uri ) 
		{
			return (DAMLProperty) Index( new DAMLPropertyImpl( uri, this, VocabularyManager.GetDefaultVocabulary() ) );
		}


		/**
		 * Create an (optionally anonymous) DAML object property.
		 *
		 * @param uri The URI for the new object property, or null to create an
		 *            anonymous object property.
		 * @return A new DAMLObjectProperty object, which is created by adding the
		 *         appropriate statements to the RDF model.
		 */
		public DAMLObjectProperty CreateDAMLObjectProperty( string uri ) 
		{
			return (DAMLObjectProperty) Index( new DAMLObjectPropertyImpl( uri, this, VocabularyManager.GetDefaultVocabulary() ) );
		}


		/**
		 * Create an (optionally anonymous) DAML datatype property.
		 *
		 * @param uri The URI for the new datatype property, or null to create
		 *            an anonymous datatype property.
		 * @return A new DAMLDatatypeProperty object, which is created by adding the
		 *         appropriate statements to the RDF model.
		 */
		public DAMLDatatypeProperty CreateDAMLDatatypeProperty( string uri ) 
		{
			return (DAMLDatatypeProperty) Index( new DAMLDatatypePropertyImpl( uri, this, VocabularyManager.GetDefaultVocabulary() ) );
		}


    /**
     * Create an (optionally anonymous) DAML datatype.
     *
     * @param uri The URI for the new datatype, or null to create
     *            an anonymous datatype.
     * @return A new DAMLDatatype object, which is created by adding the
     *         appropriate statements to the RDF model.
     */

		public DAMLDatatype CreateDAMLDatatype( string uri ) 
		{
			return (DAMLDatatype) Index( new DAMLDatatypeImpl( uri, this, VocabularyManager.GetDefaultVocabulary() ) );
		}



		/**
		 * Create an (optionally anonymous) DAML list.
		 *
		 * @param uri The URI for the new list, or null to create
		 *            an anonymous list.
		 * @return A new DAMLList object, which is created by adding the
		 *         appropriate statements to the RDF model.
		 */
		public DAMLList CreateDAMLList( string uri ) 
		{
			return (DAMLList) Index( new DAMLListImpl( uri, this, VocabularyManager.GetDefaultVocabulary() ) );
		}



		/**
		 * Create an (optionally anonymous) DAML Restriction.
		 *
		 * @param uri The URI for the new restriction, or null to create
		 *            an anonymous restriction.
		 * @return A new DAMLRestriction object, which is created by adding the
		 *         appropriate statements to the RDF model.
		 */

			public DAMLRestriction CreateDAMLRestriction( string uri ) {
				return (DAMLRestriction) Index( new DAMLRestrictionImpl( uri, this, VocabularyManager.GetDefaultVocabulary() ) );
			}



		/**
		 * Create a new DAML value that is a member of the given class.  The appropriate
		 * DAMLCommon sub-class will be instantiated, so, for example, if the DAMLClass
		 * is {@link com.hp.hpl.jena.vocabulary.DAML_OIL#Restriction}, a {@link DAMLRestriction}
		 * object will be returned.  Note that if a URI is given, and a value with that
		 * URI already exists in the model, that instance will be returned instead of
		 * creating a new DAML value. This is necessary to maintain consistency of the model.
		 *
		 * @param uri The URI of the new DAML value, or null for an anonymous value
		 * @param damlClass The class to which the new DAML value will belong
		 * @param vocabulary The vocabulary to use for the new value, or null to use
		 *                   the default vocabulary.
		 * @return an instance of a DAMLCommon value that corresponds to the given class.
		 */
		public DAMLCommon CreateDAMLValue( string uri, Resource damlClass, DAMLVocabulary vocabulary ) 
		{
			bool found = false;
			DAMLCommon instance = null;

			// see if we can match the DAML class to a known type
			for (int i = 0;  !found  &&  i < DAML_CLASS_TABLE.Length;  i++) 
			{
				Resource dClass = (Resource) DAML_CLASS_TABLE[i,0];
				Type jClass = (Type) Type.GetType((string)DAML_CLASS_TABLE[i,1]);

				if (dClass.Equals( damlClass )) 
				{
					// found a match
					found = true;

					// make the instance
					try 
					{
						// get the constructor we want to invoke and make a new instance
						/*
						instance = (DAMLCommon) jClass.GetDeclaredConstructor( DAMLLoader.s_constructSig )
													  .newInstance( new Object[] {uri, this, VocabularyManager.GetVocabulary( damlClass.GetURI() )} );
						*/
						instance  =  (DAMLCommon)Activator.CreateInstance(jClass, new object[] {uri, this, VocabularyManager.GetVocabulary( damlClass.GetURI() )});

					}
					catch (Exception e) 
					{
						Console.Error.WriteLine( "Failed to construct DAML value " + jClass.FullName );
						throw new System.Exception( "Unexpected error while constructing DAML value from RDF model: " + e );
					}
				}
			}

			// if we get here without finding a match, assume it's an instance
			if (!found  &&  (damlClass is DAMLClass)) 
			{
				instance = new DAMLInstanceImpl( uri, this, VocabularyManager.GetVocabulary( damlClass.GetURI() ) );
			}

			// ensure that we record the type of the new value in the model
			try 
			{
				if (instance != null) 
				{
					Add( instance, BBN.RDF.Vocabulary.RDF.type, damlClass );

					// index the new value
					Index( instance );
				}
			}
			catch (System.Exception e) 
			{
				Console.Error.WriteLine( "RDFException while adding statment to model: " + e );
			}

			return instance;
		}



    /**
     * <p>
     * Add all of the statements from the given model into this model. This
     * method has been extended over the standard behaviour in {@link Model#add( Model )},
     * in that it will perform the necessary processing to recognise the DAML values
     * that are present as a result of the merge.
     * </p><p>
     * <b>NB</b> this method does not recognise and process <code>daml:imports</code>
     * statements, so only the statements from the given model will be loaded.
     * </p>
     *
     * @param model A model whose statements will be added to this DAMLModel
     * @return This model
     */

    public override Model Add( Model model ) {
        GetLoader().Add( model );
        return this;
    }


		/**
		 * Answer the DAML value that corresponds to the given URI, if it exists in the
		 * model.  If the URI does not match any of the resources presently in the model,
		 * null is returned.
		 *
		 * @param uri The URI of the DAML resource to look for.
		 * @return An existing DAML resource from the model, matching uri, or null if
		 *         no such resource is found.
		 */
		public DAMLCommon GetDAMLValue( string uri ) 
		{
			Resource res = null;

			// work-around a bug in Model.GetResource(), which will create a new resource
			// if one is not found in the model
			if (ContainsResource( uri )) 
			{
				try 
				{
					res = GetResource( uri );
				}
				catch (System.Exception e) 
				{
					Console.Error.WriteLine( "RFD exception while getting resource: " + e);
				}

				// check if the resource corresponds to a DAML value
				if (res != null  &&  res is DAMLCommon) 
				{
					return (DAMLCommon) res;
				}
			}

			return null;
		}



		/**
		 * Answer the DAML value that corresponds to the given URI, if it exists in the
		 * model.  If the URI does not match any of the resources presently in the model,
		 * create a new DAML resource with the given URI and vocabulary, from the given
		 * DAML class.
		 *
		 * @param uri The URI of the DAML resource to look for.
		 * @param damlClass The class of the new resource to create if no existing resource
		 *                  is found.
		 * @param vocabulary The vocabulary to use for the new value (if needed), or null
		 *                   to use the default vocabulary.
		 * @return An existing DAML resource from the model, matching uri, or a new
		 *         resource if no existing resource is found.
		 */
		public DAMLCommon GetDAMLValue( string uri, DAMLClass damlClass, DAMLVocabulary vocabulary ) 
		{
			DAMLCommon res = GetDAMLValue( uri );

			if (res == null  &&  damlClass != null) 
			{
				return CreateDAMLValue( uri, damlClass, vocabulary );
			}
			else 
			{
				return res;
			}
		}



		/**
		 * Answer an iterator over all DAML classes that are presently in the model.
		 *
		 * @return an iterator over all currently defined classes (including Restrictions).
		 */
		public IEnumerator ListDAMLClasses() 
		{
			return GetIndex( DAML_OIL._Class.GetURI() ).GetEnumerator();
		}


		/**
		 * Answer an iterator over all DAML properties that are presently in the model.
		 *
		 * @return an iterator over all currently defined properties (i.e. rdf:Property and
		 *         all sub-classes).
		 */

		public IEnumerator ListDAMLProperties() 
		{
			return GetIndex( DAML_OIL._Property.GetURI() ).GetEnumerator();
		}



		/**
		 * Answer an iterator over all DAML instances that are presently in the model.
		 *
		 * @return an iterator over all currently defined DAML instances.
		 */
		public IEnumerator ListDAMLInstances() 
		{
			return GetIndex( DAML_OIL._Thing.GetURI() ).GetEnumerator();
		}


		/**
		 * Read the ontology indicated by the given uri.  Note that, depending on the settings in the
		 * embedded {@link DAMLLoader}, ontology import statements embedded in this document will be
		 * processed and the ontologies fetched and loaded.
		 *
		 * @param uri The URI identifying an ontology to be added, which is assumed to be
		 *            represented in the XML serialisation of RDF.
		 * @return self.
		 */
		/* 
			public Model Read( string uri ) {
				return Read( uri, uri, null );
			}
		*/


		/**
		 * Read the ontology indicated by the given uri.  Note that, depending on the settings in the
		 * embedded {@link DAMLLoader}, ontology import statements embedded in this document will be
		 * processed and the ontologies fetched and loaded.
		 *
		 * @param uri The URI identifying an ontology to be added, which is assumed to be
		 *            represented in the XML serialisation of RDF.
		 * @param lang The encoding language of the source document
		 * @return self.
		 */
		/*
			public Model Read( string uri, string lang ) {
				return Read( uri, uri, lang );
			}
		*/


		/**
		 * Read the ontology from the given reader, assuming that its base URI is
		 * the given URI.  Note that, depending on the settings in the
		 * embedded {@link DAMLLoader}, ontology import statements embedded in this document will be
		 * processed and the ontologies fetched and loaded.
		 *
		 * @param in A reader, from which will be read the DAML definitions, which are assumed to be
		 *            represented in the XML serialisation of RDF.
		 * @param base The base URI for any relative definitions that are loaded.
		 * @return self.
		 */
		/*
			public Model Read( Reader in, string base ) {
				return Read( in, base, null );
			}
		*/



		/**
		 * Read the ontology indicated by the given uri.  Note that, depending on the settings in the
		 * embedded {@link DAMLLoader}, ontology import statements embedded in this document will be
		 * processed and the ontologies fetched and loaded.
		 *
		 * @param uri The URI identifying an ontology to be added.
		 * @param base The base URI for any relative names that are loaded from the source document
		 * @param lang Denotes the language the statements are represented in.
		 * @return self.
		 * @see com.hp.hpl.mesa.rdf.jena.model.Model#read( java.lang.string, java.lang.string )
		 */

		public Model Read( string uri, string b, string lang ) 
		{
			// clear the status flag
			GetLoader().ResetStatus();
			GetLoader().Read( uri, b, lang );

			return this;
		}

		public override Model Read(System.IO.TextReader reader, string b) 
		{
			return Read(reader, b, null);
		}

		public override Model Read(System.IO.TextReader reader, string b, string lang)
		{
			GetLoader().ResetStatus();
			GetLoader().Read(reader, b, lang);
			return this;
		}

		public override Model Read(System.IO.Stream stream, string b)
		{
			return Read(stream, b, null);
		}

		public override Model Read(System.IO.Stream stream, string b, string lang)
		{
			GetLoader().ResetStatus();
			GetLoader().Read(stream, b, lang);
			return this;
		}

		public override Model Read(string url)
		{
			//DER don't base base uri if it is not different
			return Read(url, null, null);
			//return Read(url, url, null);
		}

		public override Model Read(string file, string b)
		{
			return Read(file, b, null);
		}

		public override Model ReadFromString(string xml, string b)
		{
			GetLoader().ResetStatus();
			GetLoader().ReadFromString(xml, b, null);
			return this;
		}



		/**
		 * Read the ontology from the given reader, assuming that its base URI is
		 * the given URI.  Note that, depending on the settings in the
		 * embedded {@link DAMLLoader}, ontology import statements embedded in this document will be
		 * processed and the ontologies fetched and loaded.  Use {@link #getLoadSuccessful} to
		 * check whether the load operation was successful.
		 *
		 * @param in A reader, from which will be read the DAML definitions.
		 * @param base The base URI for any relative definitions that are loaded.
		 * @param lang Denotes the language the statements are represented in.
		 * @return self.
		 */
		/* 
			public Model Read( Reader in, string base, string lang ) {
				// clear the status flag
				GetLoader().ResetStatus();

				try {
					// load the descriptions, and place them into the this model
					GetLoader().Read( in, base, lang );

					// check the loader status, to see if we were successful
					if (!GetLoadSuccessful()) {
						Console.Error.WriteLine( "Errors reported during model load: check log file for details, status code = " + GetLoader().GetStatus() );
					}
				}
				catch (RuntimeException e) {
					Console.Error.WriteLine( "Saw runtime exception: " + e );
				}

				return this;
			}
		*/



		/**
		 * Answer the DAML Loader that is used by this store to load DAML source
		 * documents.  Fine control over loading policy, such as whether imported
		 * ontologies are automatically loaded, are controlled by the loader class.
		 *
		 * @return a reference to this model's loader.
		 */
			public DAMLLoader GetLoader() {
				return m_loader;
			}



		/**
		 * Answer true if the most recent load operation was successful. If not,
		 * consult {@link DAMLLoader#GetStatus} for details, and check error log.
		 *
		 * @return true if the most recent model load was successful
		 */

			public bool GetLoadSuccessful() {
				return GetLoader().GetStatus() == DAMLLoader.STATUS_OK;
			}



		/**
		 * Answer a reference to the XML datatype registry for this model, that can be used to
		 * map between XML data marked up using XML Schema data descriptions, and Java objects.
		 * This registry is also used to detect resources in the model that correspond to
		 * {@link com.hp.hpl.jena.daml.DAMLDataInstance} objects, and type declarations that correspond to
		 * {@link com.hp.hpl.jena.daml.DAMLDatatype} objects.
		 *
		 * @return an XML data translator registry
		 */
			public XMLDatatypeRegistry GetDatatypeRegistry() {
				return m_xmlDatatypeRegistry;
			}



		/**
		 * Add an existing DAML object to the indexes maintained by this store.
		 *
		 * @param damlObj An object impplementing the DAML common interface.
		 * @return Returns the stored object, to allow chained method calls.
		 */
		public DAMLCommon Index( DAMLCommon damlObj ) 
		{
			// Get the key for the object
			Object key = ((DAMLCommonImpl) damlObj).GetKey();

			// Get the index for this key type and add the value
			GetIndex( key ).Add( damlObj );

			return damlObj;
		}


		/**
		 * Remove an existing DAML object from the indexes maintained by this store.
		 *
		 * @param damlObj An object impplementing the DAML common interface.
		 */
		public void Unindex( DAMLCommon damlObj ) 
		{
			// Get the key for the object
			Object key = ((DAMLCommonImpl) damlObj).GetKey();

			// Get the index for this key type and add the value
			GetIndex( key ).Remove( damlObj );
		}


		/**
		 * Flag to control whether accessing the DAML store will take into account equivalence classes for
		 * properties and resources, using <code>daml:equivalentTo</code> and similar
		 * statements.  Turning this flag on is correct according to the DAML semantics, but
		 * will impose a significant performance problem in the current version.  Turning
		 * the flag off will improve performance, at the cost of not conforming strictly
		 * to the DAML specification.
		 *
		 * @param useEquivalence If true, accessing properties and resources will check for
		 *                       equivalent values, at a cost of reduced performance.
		 */
		/**
		 * Answer true if the model will consider equivalence classes when accessing
		 * properties and resources.  See {@link #SetUseEquivalence} for details.
		 *
		 * @return true if equivalence classes are being considered.
		 */
		public bool UseEquivalence
		{
			set 
			{
				m_useEquivalence = value;
			}
			get
			{
				return m_useEquivalence;
			}
		}




		// Internal implementation methods
		//////////////////////////////////

		/**
		 * Initialise the store with well-known values.
		 */
		protected void InitStore() 
		{}



		/**
		 * Answer the index that corresponds to the given key.
		 *
		 * @param key The key object
		 * @return the corresponding
		 */

		protected ArrayList GetIndex( Object key ) 
		{
			ArrayList index = (ArrayList) m_indexes[ key ];
			if (index == null) 
			{
				index = new ArrayList();
				m_indexes[ key ] = index;
			}

			return index;
		}



		/**
		 * Answer true if the model contains the given resource, in any position
		 * in a triple (subject, predicate or object).
	     *
	     * @param uri The URI of the resource, as a string
	     * @return True if at least one statement with a resource with the given URI
	     *         as a subject, predicate or object is currently in the model.
		 */
		protected bool ContainsResource( string uri ) 
		{
			try 
			{
				// first try as a subject
				Resource r = GetResource( uri );
				StmtEnumerator i0 = ListStatements( new SelectorImpl( r, null, (RDFNode) null ) );
				if (i0.MoveNext()) 
				{
					return true;
				}

				// now as object
				StmtEnumerator i1 = ListStatements( new SelectorImpl( null, null, r ) );
				if (i1.MoveNext()) 
				{
					return true;
				}

				// now as predicate: note that this URI may not be a valid predicate URI
				// so we ignore any RDFException in this region
				try 
				{
					Property p = GetProperty( uri );
					StmtEnumerator p0 = ListStatements( new SelectorImpl( null, p, (RDFNode) null ) );
					if (p0.MoveNext()) 
					{
						return true;
					}
				}
				catch (System.Exception) {}

			}
			catch (System.Exception e) 
			{
				Console.Error.WriteLine( "RDF exception: " + e );
			}
			// Not in the Model
			return false;
		}




		//==============================================================================
		// Inner class definitions
		//==============================================================================

	}
}
