using System;
using BBN.DAML.Vocabulary;

namespace BBN.DAML
{
	/// <summary>
	/// 
	/// </summary>
	public class DAMLRestrictionImpl : DAMLClassImpl, DAMLRestriction
	{


		// Instance variables
		//////////////////////////////////

		/** Property accessor for onProperty */
		private PropertyAccessor m_propOnProperty = null;

		/** Property accessor for hasClass */
		private PropertyAccessor m_propHasClass = null;

		/** Property accessor for toClass */
		private PropertyAccessor m_propToClass = null;

		/** Property accessor for hasValue */
		private PropertyAccessor m_propHasValue = null;

		/** Property accessor for hasClassQ */
		private PropertyAccessor m_propHasClassQ = null;

		/** Property accessor for cardinality */
		private IntLiteralAccessor m_propCardinality = null;

		/** Property accessor for minCardinality */
		private IntLiteralAccessor m_propMinCardinality = null;

		/** Property accessor for maxCardinality */
		private IntLiteralAccessor m_propMaxCardinality = null;

		/** Property accessor for cardinalityQ */
		private IntLiteralAccessor m_propCardinalityQ = null;

		/** Property accessor for minCardinalityQ */
		private IntLiteralAccessor m_propMinCardinalityQ = null;

		/** Property accessor for maxCardinalityQ */
		private IntLiteralAccessor m_propMaxCardinalityQ = null;



		// Constructors
		//////////////////////////////////

		/**
		 * Constructor, takes the name and namespace for this restriction, and the underlying
		 * model it will be attached to.
		 *
		 * @param namespace The namespace the restriction inhabits, or null
		 * @param name The name of the restriction
		 * @param store The RDF store that contains the RDF statements defining the properties of the restriction
		 * @param vocabulary Reference to the DAML vocabulary used by this restriction.
		 */
		public DAMLRestrictionImpl( string ns, string name, DAMLModel store, DAMLVocabulary vocabulary )
			:base( ns, name, store, vocabulary )
		{
			SetRDFType( GetVocabulary().Restriction );
		}


		/**
		 * Constructor, takes the URI for this restriction, and the underlying
		 * model it will be attached to.
		 *
		 * @param uri The URI of the restriction
		 * @param store The RDF store that contains the RDF statements defining the properties of the restriction
		 * @param vocabulary Reference to the DAML vocabulary used by this restriction.
		 */
		public DAMLRestrictionImpl( string uri, DAMLModel store, DAMLVocabulary vocabulary )
			:base( uri, store, vocabulary )
		{
			
			SetRDFType( GetVocabulary().Restriction );
		}



		// External signature methods
		//////////////////////////////////


		/**
		 * Answer true if this class expression is an enumeration (i.e. has a property
		 * 'oneOf' with a list of values).  This is not an exclusive property, a class
		 * expression can be an enumeration at the same time as one of the other kinds
		 * of class expression, though the conjunction of these may produce the Nothing
		 * class.
		 *
		 * @return necessarily false, since enumerations only appear in {@link com.hp.hpl.jena.daml.DAMLClass}
		 *         expressions.
		 */
		public override bool IsEnumeration
		{
			get 
			{
				return false;
			}
		}


		/**
		 * Answer true if this class expression is an named class (i.e. is not an anonymous
		 * class expression).  This is not an exclusive property, a class
		 * expression can be named at the same time as one of the other kinds
		 * of class expression, though the conjunction of these may produce the Nothing
		 * class.
		 *
		 * @return necessarily false, since restrictions are necessarily constructed
		 *         expressions. Note: this is not the same as testing whether the restriction
		 *         itself has a name (so that it can be referenced elsewhere).
		 */
		public override bool IsNamedClass
		{
			get 
			{
				return false;
			}
		}


		/**
		 * Answer true if this class expression is an property restriction (i.e. is a
		 * Restriction value).  This is not an exclusive property, a class
		 * expression can be a property restriction at the same time as one of the other kinds
		 * of class expression, though the conjunction of these may produce the Nothing
		 * class.
		 *
		 * @return necessarily true.
		 */
		public override bool IsRestriction
		{
			get 
			{
				return true;
			}
		}


		/**
		 * Answer true if this class expression is an boolean intersection of a list
		 * of class expressions.  This is not an exclusive property, a class
		 * expression can be an intersection at the same time as one of the other kinds
		 * of class expression, though the conjunction of these may produce the Nothing
		 * class.
		 *
		 * @return necessarily false, since interesections only appear in {@link com.hp.hpl.jena.daml.DAMLClass}
		 *         expressions.
		 */
		public override bool IsIntersection
		{
			get 
			{
				return false;
			}
		}


		/**
		 * Answer true if this class expression is an boolean union of a list
		 * of class expressions.  This is not an exclusive property, a class
		 * expression can be an union at the same time as one of the other kinds
		 * of class expression, though the conjunction of these may produce the Nothing
		 * class.
		 *
		 * @return necessarily false, since unions only appear in {@link com.hp.hpl.jena.daml.DAMLClass}
		 *         expressions.
		 */
		public override bool IsUnion
		{
			get 
			{
				return false;
			}
		}


		/**
		 * Answer true if this class expression is an boolean complement of a list
		 * of class expressions.  This is not an exclusive property, a class
		 * expression can be an complement at the same time as one of the other kinds
		 * of class expression, though the conjunction of these may produce the Nothing
		 * class.
		 *
		 * @return necessarily false, since complements only appear in {@link com.hp.hpl.jena.daml.DAMLClass}
		 *         expressions, as opposed to compliments, which you only get on your birthday.
		 */
		public override bool  IsComplement
		{
			get 
			{
				return false;
			}
		}


		/**
		 * Property accessor for the 'onProperty' property of a restriction. This
		 * denotes the property to which the restriction applies, and there is normally
		 * exactly one of them.
		 *
		 * @return Property accessor for 'onProperty'.
		 */
		public PropertyAccessor Prop_OnProperty
		{
			get 
			{
				if (m_propOnProperty == null) 
				{
					m_propOnProperty = new PropertyAccessorImpl( GetVocabulary().onProperty, this );
				}

				return m_propOnProperty;
			}
		}


		/**
		 * Property accessor for the 'toClass' property of a restriction. This denotes
		 * the class for which the restricted property always maps to instances that
		 * belong to the class given by this property.
		 *
		 * @return Property accessor for 'toClass'
		 */
		public PropertyAccessor Prop_ToClass
		{
			get 
			{
				if (m_propToClass == null) 
				{
					m_propToClass = new PropertyAccessorImpl( GetVocabulary().toClass, this );
				}

				return m_propToClass;
			}
		}


		/**
		 * Property accessor for the 'hasValue' property of a restriction. This denotes
		 * the class for which the restricted property sometimes maps to the instance given
		 * here.
		 *
		 * @return Property accessor for 'hasValue'
		 */
		public PropertyAccessor Prop_HasValue
		{
			get 
			{
				if (m_propHasValue == null) 
				{
					m_propHasValue = new PropertyAccessorImpl( GetVocabulary().hasValue, this );
				}

				return m_propHasValue;
			}
		}


		/**
		 * Property accessor for the 'hasClass' property of a restriction. This denotes
		 * the class for which the restricted property sometimes maps to the instances that
		 * belong to the class given here.
		 *
		 * @return Property accessor for 'hasClass'
		 */
		public PropertyAccessor Prop_HasClass
		{
			get 
			{
				if (m_propHasClass == null) 
				{
					m_propHasClass = new PropertyAccessorImpl( GetVocabulary().hasClass, this );
				}

				return m_propHasClass;
			}
		}


		/**
		 * Property accessor for the 'hasClassQ' property of a restriction. This denotes
		 * the class for which the restricted property sometimes maps to the instances that
		 * belong to the class given here, and which obey given cardinality constraints.
		 *
		 * @return Property accessor for 'hasClassQ'
		 */
		public PropertyAccessor Prop_HasClassQ
		{
			get 
			{
				if (m_propHasClassQ == null) 
				{
					m_propHasClassQ = new PropertyAccessorImpl( GetVocabulary().hasClassQ, this );
				}

				return m_propHasClassQ;
			}
		}


		/**
		 * Property accessor for the 'cardinality' property of a restriction. This denotes
		 * the combination of minCardinality and maxCardinality to the same value.
		 *
		 * @return Property accessor for 'cardinality'
		 */
		public IntLiteralAccessor Prop_Cardinality
		{
			get 
			{
				if (m_propCardinality == null) 
				{
					m_propCardinality = new IntLiteralAccessorImpl( GetVocabulary().cardinality, this );
				}

				return m_propCardinality;
			}
		}


		/**
		 * Property accessor for the 'minCardinality' property of a restriction. This denotes
		 * the class of instances that have at least N distict values for the property.
		 *
		 * @return Property accessor for 'minCardinality'
		 */
		public IntLiteralAccessor Prop_MinCardinality
		{
			get 
			{
				if (m_propMinCardinality == null) 
				{
					m_propMinCardinality = new IntLiteralAccessorImpl( GetVocabulary().minCardinality, this );
				}

				return m_propMinCardinality;
			}
		}


		/**
		 * Property accessor for the 'maxCardinality' property of a restriction. This denotes
		 * the class of instances that have at most N distict values for the property.
		 *
		 * @return Property accessor for 'maxCardinality'
		 */
		public IntLiteralAccessor Prop_MaxCardinality
		{
			get 
			{
				if (m_propMaxCardinality == null) 
				{
					m_propMaxCardinality = new IntLiteralAccessorImpl( GetVocabulary().maxCardinality, this );
				}

				return m_propMaxCardinality;
			}
		}


		/**
		 * Property accessor for the 'cardinalityQ' property of a restriction. This denotes
		 * the combination of minCardinalityQ and maxCardinalityQ to the same value.
		 *
		 * @return Property accessor for 'cardinalityQ'
		 */
		public IntLiteralAccessor Prop_CardinalityQ
		{
			get 
			{
				if (m_propCardinalityQ == null) 
				{
					m_propCardinalityQ = new IntLiteralAccessorImpl( GetVocabulary().cardinalityQ, this );
				}

				return m_propCardinalityQ;
			}
		}


		/**
		 * Property accessor for the 'minCardinalityQ' property of a restriction. This denotes
		 * the class of instances that have at least N distict values of the class denoted by
		 * 'hasClassQ' for the property.
		 *
		 * @return Property accessor for 'minCardinalityQ'
		 */
		public IntLiteralAccessor Prop_MinCardinalityQ
		{
			get 
			{
				if (m_propMinCardinalityQ == null) 
				{
					m_propMinCardinalityQ = new IntLiteralAccessorImpl( GetVocabulary().minCardinalityQ, this );
				}

				return m_propMinCardinalityQ;
			}
		}


		/**
		 * Property accessor for the 'maxCardinalityQ' property of a restriction. This denotes
		 * the class of instances that have at most N distict values of the class denoted by
		 * 'hasClassQ' for the property.
		 *
		 * @return Property accessor for 'maxCardinalityQ'
		 */
		public IntLiteralAccessor Prop_MaxCardinalityQ
		{
			get 
			{
				if (m_propMaxCardinalityQ == null) 
				{
					m_propMaxCardinalityQ = new IntLiteralAccessorImpl( GetVocabulary().maxCardinalityQ, this );
				}

				return m_propMaxCardinalityQ;
			}
		}


		/**
		 * Answer a key that can be used to index collections of this DAML class for
		 * easy access by iterators.  Package access only.
		 *
		 * @return a key object.
		 */
		public override Object GetKey() 
		{
			return DAML_OIL._Class.GetURI();
		}


	}
}
