using System;
using System.Collections;
using System.Diagnostics;
using NUnit.Framework;
using BBN.DAML;
using BBN.RDF;
using BBN.DAML.Vocabulary; 

namespace BBN.DAML.Test
{
	/// <summary>
	/// 
	/// </summary>
	public class DAMLTest : TestCase
	{ 



		// Instance variables
		//////////////////////////////////

		/** True only for the first test in the sequence */
		private bool m_firstRun = true;

		// The root directory for test files used herein.
		private string testFiles = "file://c:/daml/test/dotnetAPI/";


		// Constructors
		//////////////////////////////////

		/**
		 * Constructor requires that all tests be named
		 *
		 * @param name The name of this test
		 */
		public DAMLTest( string name ) 
			: base(name) 
		{
		}



		// External signature methods
		//////////////////////////////////

		/**
		 * Answer a suite of all the tests defined here
		 */
		public static ITest Suite
		{
			get
			{
				TestSuite suite = new TestSuite();

				// add all the tests defined in this class to the suite
				/* */  suite.AddTest( new DAMLTest( "TestCollection" ) ); /* */
				/* */  suite.AddTest( new DAMLTest( "TestLoadOntology" ) );   /*  */
				/*  suite.AddTest( new DAMLTest( "TestRDFType" ) );          */
				/*  suite.AddTest( new DAMLTest( "TestClass" ) );            */
				/*  suite.AddTest( new DAMLTest( "TestEquivalence" ) );      */
				/*  suite.AddTest( new DAMLTest( "TestProperty" ) );         */
				/*  suite.AddTest( new DAMLTest( "TestList" ) );             */
				/*  suite.AddTest( new DAMLTest( "TestDatatype" ) );         */
				/*  suite.AddTest( new DAMLTest( "TestInstance" ) );         */
				/*  suite.AddTest( new DAMLTest( "TestRemove" ) );           */
				/*  suite.AddTest( new DAMLTest( "TestCreate" ) );           */
				/*  suite.AddTest( new DAMLTest( "TestRestriction" ) );      */
				/*  suite.AddTest( new DAMLTest( "TestModelAdd" ) );         */
				/*  suite.AddTest( new DAMLTest( "TestSyntaxError" ) );      */

				// tests arising from Jeremy's tutorial
				/*  suite.AddTest( new DAMLTest( "TestDatatypeProperty" ) );     */
				/*  suite.AddTest( new DAMLTest( "TestPropertyEq" ) );     */
				/*  suite.AddTest( new DAMLTest( "TestObjectProperty" ) );     */
				/*  suite.AddTest( new DAMLTest( "TestDatatypeEq1" ) );     */
				/*  suite.AddTest( new DAMLTest( "TestDatatypeEq2" ) );     */
				/*  suite.AddTest( new DAMLTest( "TestDatatypeRange" ) );     */

				return suite;
			}
		}


		// Internal implementation methods
		//////////////////////////////////

		/**
		 * Set up the test conditions
		 */
		protected override void SetUp() 
		{
			//Log.getInstance().setLevel( Log.FINEST );
			/*
			try 
			{
				Log.getInstance().SetDefaultFileHandler( !m_firstRun );
			}
			catch (Exception e) 
			{
				System.err.println( "Could not open log file " + e );
			}
			Log.finest( "Test suite setting up" );
			*/
			m_firstRun = false;
		}


		/**
		 * Release objects no longer needede when we're done
		 */
		protected override void TearDown() 
		{
			//Log.debug( "Test suite tearing down" );
		}



		// Test cases
		/////////////

		/**
		 * Test the various pathways through loading the ontology from a source document.
		 */

		public void TestCollection()
		{
			DAMLModel m = new DAMLModelImpl();
			m.Read(testFiles+"collection1.daml");
			Assertion.Assert("collection1.daml Load success status should be true", m.GetLoadSuccessful() );
			//Console.WriteLine(m.Count);
			//Console.WriteLine(countClasses(m));
			//Console.WriteLine(countProperties(m));
			AssertEquals("collection1.daml Count of statements", 24, (int)m.Count);
			AssertEquals("collection1.daml Count of Classes", 5, countClasses(m));
			AssertEquals("collection1.daml Count of Properties", 0, countProperties(m));
			m = new DAMLModelImpl();
			m.Read(testFiles+"collection2.daml");
			Assertion.Assert("collection2.daml Load success status should be true", m.GetLoadSuccessful() );
			//Console.WriteLine(m.Count);
			//Console.WriteLine(countClasses(m));
			//Console.WriteLine(countProperties(m));
			AssertEquals("collection2.daml Count of statements", 12, (int)m.Count);
			AssertEquals("collection2.daml Count of Classes", 3, countClasses(m));
			AssertEquals("collection2.daml Count of Properties", 0, countProperties(m));
			m = new DAMLModelImpl();
			m.Read(testFiles+"collection3.daml");
			Assertion.Assert("collection3.daml Load success status should be true", m.GetLoadSuccessful() );
			//Console.WriteLine(m.Count);
			//Console.WriteLine(countClasses(m));
			//Console.WriteLine(countProperties(m));
			AssertEquals("collection3.daml Count of statements", 25, (int)m.Count);
			AssertEquals("collection3.daml Count of Classes", 5, countClasses(m));
			AssertEquals("collection3.daml Count of Properties", 0, countProperties(m));
			m = new DAMLModelImpl();
			m.Read(testFiles+"collection4.daml");
			Assertion.Assert("collection4.daml Load success status should be true", m.GetLoadSuccessful() );
			//Console.WriteLine(m.Count);
			//Console.WriteLine(countClasses(m));
			//Console.WriteLine(countProperties(m));
			AssertEquals("collection4.daml Count of statements", 25, (int)m.Count);
			AssertEquals("collection4.daml Count of Classes", 5, countClasses(m));
			AssertEquals("collection4.daml Count of Properties", 0, countProperties(m));
			m = new DAMLModelImpl();
			m.Read(testFiles+"collection5.daml");
			Assertion.Assert("collection5.daml Load success status should be true", m.GetLoadSuccessful() );
			//Console.WriteLine(m.Count);
			//Console.WriteLine(countClasses(m));
			//Console.WriteLine(countProperties(m));
			AssertEquals("collection5.daml Count of statements", 25, (int)m.Count);
			AssertEquals("collection5.daml Count of Classes", 5, countClasses(m));
			AssertEquals("collection5.daml Count of Properties", 0, countProperties(m));
		}

		public void TestLoadOntology()
		{
			//        Log.debug( "Starting loadOntology tests" );

			DAMLModel m = new DAMLModelImpl();

			// first do the model read with all options turned on
			//        Log.debug( "Test: loading (2000/12, import, standard block)" );
			string f = testFiles+"daml_oil_2000_12/daml+oil-ex.daml";
			m.Read( f, "http://www.daml.org/2000/12/daml+oil-ex", null );
			//Console.WriteLine(f);
			//Console.WriteLine(m.Count);
			//Console.WriteLine(countClasses(m));
			//Console.WriteLine(countProperties(m));
			Console.WriteLine(countInstances(m));
			dumpModel(m, "/daml/test/model-net.out", false);
			Assertion.Assert( f+" Load success status should be true", m.GetLoadSuccessful() );
			// Note: Jena makes explicit in statements that the four datatypes (eg xsd:integer) 
			// are of type daml:Datatype, accounting for four more statements. (221 v 217)
			Assertion.AssertEquals( f+" Count of statements", 217, (int)m.Count);
			Assertion.AssertEquals( f+" Count of classes", 28, countClasses( m ) ); 
			Assertion.AssertEquals( f+" Count of properties", 12, countProperties( m ) ); 
			//dumpModel( m );

			// now turn off importing - should only get the classes and properties in the source doc
			// Note, the example file does not import any additional files (besides DAML+OIL lang spec
			// which is not loaded by Jena.
			m = new DAMLModelImpl();
			m.GetLoader().SetLoadImportedOntologies( false );
			f = testFiles+"daml_oil_2000_12/daml+oil-ex.daml";
			m.Read( f, "http://www.daml.org/2000/12/daml+oil-ex", null );
			//Console.WriteLine(f + "(no import)");
			//Console.WriteLine(m.Count);
			//Console.WriteLine(countClasses(m));
			//Console.WriteLine(countProperties(m));
			Console.WriteLine(countInstances(m));
			Assertion.Assert( f+" Load success status should be true", m.GetLoadSuccessful() );
			Assertion.AssertEquals( f+" (no import) Count of statements", 217, (int)m.Count); // 221 in Jena (see above)
			Assertion.AssertEquals( f+" (no import) Count of classes", 28, countClasses( m ) ); 
			Assertion.AssertEquals( f+" (no import) Count of properties", 8, countProperties( m ) ); 

			// try again, this time we'll block the loading of our local extension
			//        Log.debug( "Test: loading (2000/12, import, added to block list)" );
			m = new DAMLModelImpl();
			m.GetLoader().AddImportBlock( testFiles+"daml_oil_2000_12/daml-local-ex.daml" );
			m.Read( testFiles+"daml_oil_2000_12/daml+oil-ex.daml", "http://www.daml.org/2000/12/daml+oil-ex", null );
			Assertion.Assert( "Load success status should be true", m.GetLoadSuccessful() );
			Assertion.AssertEquals( "Count of number of classes in daml store (2000/12, import block)", 34, countClasses( m ) );
			Assertion.AssertEquals( "Count of number of properties in daml store (2000/12, import block)", 44, countProperties( m ) );

			// repeat with 2001/03 version

			// first do the model read with all options turned on
			//        Log.debug( "Test: loading (2001/03, import, standard block)" );
			m = new DAMLModelImpl();
			m.Read( testFiles+"daml_oil_2001_03/daml+oil-ex.daml", "http://www.daml.org/2001/03/daml+oil-ex", null );
			Assertion.Assert( "Load success status should be true", m.GetLoadSuccessful() );
			Assertion.AssertEquals( "Count of number of classes in daml store (2001/03, import)", 43, countClasses( m ) );
			Assertion.AssertEquals( "Count of number of properties in daml store (2001/03, import) ", 50, countProperties( m ) );
			//dumpModel( m );

			// now turn off importing - should only get the classes and properties in the source doc
			//        Log.debug( "Test: loading (2001/03, no import)" );
			m = new DAMLModelImpl();
			m.GetLoader().SetLoadImportedOntologies( false );
			m.Read( testFiles+"daml_oil_2001_03/daml+oil-ex.daml", "http://www.daml.org/2001/03/daml+oil-ex", null );
			Assertion.Assert( "Load success status should be true", m.GetLoadSuccessful() );
			Assertion.AssertEquals( "Count of number of classes in daml store (2001/03, no import)", 28, countClasses( m ) );
			Assertion.AssertEquals( "Count of number of properties in daml store (2001/03, no import)", 12, countProperties( m ) );

			// now we'll try to read an HTTP document, but only if the http proxy is set
			if (true) //(System.GetProperty( "proxySet" ) != null  &&  System.GetProperty( "proxySet" ).Equals( "true" )) 
			{
				//            Log.debug( "Test: loading (2001/03, http)" );
				m = new DAMLModelImpl();
				m.Read( "http://www.daml.org/2001/03/daml+oil-ex.daml", "http://www.daml.org/2001/03/daml+oil-ex", null );
				Assertion.Assert( "Load success status should be true", m.GetLoadSuccessful() );
				Assertion.AssertEquals( "Count of number of classes in daml store (2001/03, http)", 28, countClasses( m ) );
				Assertion.AssertEquals( "Count of number of properties in daml store (2001/03, http)", 12, countProperties( m ) );
			}
			else 
			{
				//            Log.finest( "HTTP proxy is not set, so skipping HTTP read test 1" );
			}

			// also try unblocking the import list so that we load the DAML ontology by http
			if (true) //(System.GetProperty( "proxySet" ) != null) 
			{
				//            Log.debug( "Test: loading (2001/03, http, block removed)" );
				m = new DAMLModelImpl();
				m.GetLoader().RemoveImportBlock( "http://www.daml.org/2001/03/daml+oil" );
				m.Read( "http://www.daml.org/2001/03/daml+oil-ex.daml", "http://www.daml.org/2001/03/daml+oil-ex", null );
				Assertion.Assert( "Load success status should be true", m.GetLoadSuccessful() );
				Assertion.AssertEquals( "Count of number of classes in daml store (2001/03, http, block removed)", 43, countClasses( m ) );
				Assertion.AssertEquals( "Count of number of properties in daml store (2001/03, http, block removed)", 50, countProperties( m ) );
			}
			else 
			{
				//            Log.finest( "HTTP proxy is not set, so skipping HTTP read test 2" );
			}

			// test case for bug reported by Charlie Abela: must be able to load instance files that import
			// their own class declarations
			m = new DAMLModelImpl();
			m.Read( testFiles+"test-instance-load.daml" );
			Assertion.Assert( "Load status should be true", m.GetLoadSuccessful() );
			Resource pugh = m.GetResource( "http://dickinson-i-4/daml/tests/test-instance-load.daml#pugh" );
			Assertion.AssertNotNull( "Resource for officer Pugh should not be null", pugh );
			Assertion.Assert( "Resource for Pugh should be recognised as a DAML instance", pugh is DAMLInstance );

			// test case for bug report by Michael Sintek
			// try to ascertain the most specific class we can at load time -
			// case in point is shoesize in standard example ontology
			m = new DAMLModelImpl();
			m.Read( testFiles+"daml_oil_2001_03/daml+oil-ex.daml", "http://www.daml.org/2001/03/daml+oil-ex", null );
			Assertion.Assert( "Load success status should be true", m.GetLoadSuccessful() );
			DAMLProperty shoesize = (DAMLProperty) m.GetProperty( "http://www.daml.org/2001/03/daml+oil-ex#shoesize" );
			Assertion.AssertNotNull( "Failed to find shoesize property in example ontology", shoesize );
			Assertion.AssertEquals( "shoesize should be a unique property", true, shoesize.Unique );
			Assertion.AssertEquals( "shoesize should be a datatype property", true, shoesize is DAMLDatatypeProperty );
		}



		/**
		 * Test case for testing rdf:type traversal
		 */
		public void TestRDFType()
		{
			string ns = "http://dickinson-i-4/daml/tests/test-cases.daml#";

			// Log.debug( "Starting rdf:type tests" );
			DAMLModel m = new DAMLModelImpl();

			// don't allow any additional info to load
			m.GetLoader().SetLoadImportedOntologies( false );
			m.Read( testFiles+"test-cases.daml", "http://dickinson-i-4/daml/tests/test-cases.daml", null );

			//dumpModel( m );

			// first find fido
			DAMLInstance fido = (DAMLInstance) m.GetDAMLValue( ns + "fido" );
			Assertion.AssertNotNull( "fido instance should not be null", fido );

			// lookup some classes for convenience
			DAMLClass cDog = (DAMLClass) m.GetDAMLValue( ns + "Dog" );
			Assertion.AssertNotNull( "Dog class should not be null", cDog );

			DAMLClass cVertebrate = (DAMLClass) m.GetDAMLValue( ns + "Vertebrate" );
			Assertion.AssertNotNull( "Vertebrate class should not be null", cVertebrate );

			DAMLClass cPet = (DAMLClass) m.GetDAMLValue( ns + "Pet" );
			Assertion.AssertNotNull( "Pet class should not be null", cPet );

			// fido is a Dog, a vertebrate and a pet
			Assertion.Assert( "fido should be member of class Dog", fido.HasRDFType( cDog ) );
			Assertion.Assert( "fido should be member of class Vertebrate", fido.HasRDFType( cVertebrate ) );
			Assertion.Assert( "fido should be member of class Vertebrate (by URL)", fido.HasRDFType( ns + "Vertebrate" ) );
			Assertion.Assert( "fido should be member of class Pet", fido.HasRDFType( cPet ) );

			// fido is not a class
			Assertion.Assert( "fido should not be a class", !fido.HasRDFType( DAML_OIL._Class ) );

			// fido is a companion, even though this class is not defined in the current ontology
			Assertion.Assert( "fido should be a companion", fido.HasRDFType( ns + "Companion" ) );

			// fido is a Thing (all things DAML are Things)
			Assertion.Assert( "fido should be a thing", fido.HasRDFType( DAML_OIL._Thing ) );

			// get some more classes
			DAMLClass cA = (DAMLClass) m.GetDAMLValue( ns + "A" );
			Assertion.AssertNotNull( "Class A should not be null", cA );

			DAMLClass cB = (DAMLClass) m.GetDAMLValue( ns + "B" );
			Assertion.AssertNotNull( "Class B should not be null", cB );

			DAMLInstance ab = (DAMLInstance) m.GetDAMLValue( ns + "ab" );
			Assertion.AssertNotNull( "Instance ab should not be null", ab );

			// note that cA --subclass--> cB --subclass--> cA   is a loop
			Assertion.Assert( "ab should be an A", ab.HasRDFType( cA ) );
			Assertion.Assert( "ab should be a B", ab.HasRDFType( cB ) );

			// how many ways do I know thee? let me count the ways ...
			Assertion.AssertEquals( "Number of classes fido belongs to (closure) should be 7",
				7, countIteration( fido.GetRDFTypes( true ), true, "fido member of class " ) );
			Assertion.AssertEquals( "Number of classes fido belongs to (non-closure) should be 2",
				2, countIteration( fido.GetRDFTypes( false ), true, "fido member of non-closed class " ) );

			// some tests on the built-in classes
			DAMLProperty queenOf = (DAMLProperty) m.GetDAMLValue( ns + "queen-of" );
			Assertion.AssertNotNull( "queen-of property should be defined", queenOf );
			Assertion.Assert( "an UnabmbiguousProperty should be an ObjectProperty", queenOf.HasRDFType( DAML_OIL._UnambiguousProperty ) );
			Assertion.Assert( "an UnabmbiguousProperty should be an ObjectProperty", queenOf.HasRDFType( DAML_OIL._ObjectProperty ) );
			Assertion.Assert( "an UnabmbiguousProperty should be an rdf:Property",   queenOf.HasRDFType( BBN.RDF.Vocabulary.RDF.Property ) );
		}


		/**
		 * Test some of the properties of DAML classes
		 */
		public void TestClass()
		{
			// Log.debug( "Starting DAML class tests" );
			DAMLModel m = new DAMLModelImpl();

			m.Read( testFiles+"daml_oil_2001_03/daml+oil-ex.daml", "http://www.daml.org/2001/03/daml+oil-ex", null );
			Assertion.Assert( "loadStatus should be true for successful load", m.GetLoadSuccessful() );
			string ns = "http://www.daml.org/2001/03/daml+oil-ex#";

			// get a reference to the Person class
			DAMLClass person = (DAMLClass) m.GetDAMLValue( ns + "Person" );
			Assertion.AssertNotNull( "Person class should not be null", person );
			Assertion.Assert( "Person should be a named class", person.IsNamedClass );

			// count the super-classes of a Person
			int sCount0 = countIteration( person.Prop_SubClassOf.GetAll( false ), true, "super-class of Person (prop_subClassOf) " );
			int sCount1 = countIteration( person.GetSuperClasses(), true, "super-class of Person (getSuperClasses) " );
			Assertion.AssertEquals( "person should have 7 super-classes (by prop_subClassOf)", 7, sCount0 );
			Assertion.AssertEquals( "person should have 9 super-classes (by getSuperClasses)", 9, sCount1 );

			// count the number of sub-classes of a Person
			Assertion.AssertEquals( "person should have 2 sub-classes", 2,
				countIteration( person.GetSubClasses(), true, "Person super-class of: " ) );

			// person is a disjoint union of Man and Woman
			Assertion.Assert( "Person should be a disjoint union", person.IsDisjointUnion );
			DAMLList mw = (DAMLList) person.Prop_DisjointUnionOf.Get();
			Assertion.AssertNotNull( "Value of disjoint union should not be null", mw );
			Assertion.AssertEquals( "Person should be a disjoint union of size 2", 2, mw.Count );

			// Female is disjoint with Male
			DAMLClass female = (DAMLClass) m.GetDAMLValue( ns + "Female" );
			Assertion.AssertNotNull( "Class Female should not be null", female );
			DAMLClass male = (DAMLClass) m.GetDAMLValue( ns + "Male" );
			Assertion.AssertNotNull( "Class Male should not be null", male );
			Assertion.Assert( "Female should be disjoint with male", female.Prop_DisjointWith.HasValue( male ) );

			// HumanBeing is the same class as Person
			DAMLClass humanBeing = (DAMLClass) m.GetDAMLValue( ns + "HumanBeing" );
			Assertion.AssertNotNull( "Class humanBeing should not be null", humanBeing );
			Assertion.Assert( "Person should be same class as HumanBeing", humanBeing.Prop_SameClassAs.HasValue( person ) );

			// TallMan is an intersection of Man and TallThing
			DAMLClass tallMan = (DAMLClass) m.GetDAMLValue( ns + "TallMan" );
			Assertion.AssertNotNull( "Class TallMan should not be null", tallMan );
			DAMLList tm = (DAMLList) tallMan.Prop_IntersectionOf.Get();
			Assertion.AssertNotNull( "Value of intersection should not be null", tm );
			Assertion.AssertEquals( "Tall man should be an intersection of size 2", 2, tm.Count );

			// Car is a complement of Person
			DAMLClass car = (DAMLClass) m.GetDAMLValue( ns + "Car" );
			Assertion.AssertNotNull( "Class Car should not be null", car );
			DAMLClass carSuper = (DAMLClass) car.GetSuperClasses().Current;
			Assertion.AssertNotNull( "Car should have a super-class", carSuper );
			Assertion.Assert( "Car super-class should be a complement", carSuper.IsComplement );
			Assertion.Assert( "Car super-class should be a complement of Person", carSuper.Prop_ComplementOf.HasValue( person ) );

			// Height is an enumeration of three values
			DAMLClass height = (DAMLClass) m.GetDAMLValue( ns + "Height" );
			Assertion.AssertNotNull( "Class Height should not be null", height );
			Assertion.Assert( "Height should be an enumeration", height.IsEnumeration );
			Assertion.AssertEquals( "Height should be an enumeration of 3 elements", 3, ((DAMLList) height.Prop_OneOf.Get()).Count );

			// daml:subClassOf is processed as rdfs:subClassOf
			DAMLModel m0 = new DAMLModelImpl();
			m0.GetLoader().SetLoadImportedOntologies( false );
			m0.Read( testFiles+"test-cases.daml", "http://dickinson-i-4/daml/tests/test-cases.daml", null );
			string tcNs = "http://dickinson-i-4/daml/tests/test-cases.daml#";
			DAMLClass subClassBug0 = (DAMLClass) m0.GetDAMLValue( tcNs + "SubClassBug0" );
			DAMLClass subClassBug1 = (DAMLClass) m0.GetDAMLValue( tcNs + "SubClassBug1" );
			Assertion.AssertNotNull( "Class SubClassBug0 should not be null", subClassBug0 );
			Assertion.AssertNotNull( "Class SubClassBug1 should not be null", subClassBug1 );
			Assertion.Assert( "SubClassBug1 should have SubClassBug0 as a super-class", subClassBug1.HasSuperClass( subClassBug0 ) );
			Assertion.Assert( "SubClassBug0 should have SubClassBug1 as a sub-class", subClassBug0.HasSubClass( subClassBug1 ) );

			// defined properties are those that mention this class as domain
			DAMLClass defProp0 = (DAMLClass) m0.GetDAMLValue( tcNs + "DefProp0" );
			DAMLClass defProp1 = (DAMLClass) m0.GetDAMLValue( tcNs + "DefProp1" );
			DAMLClass defProp2 = (DAMLClass) m0.GetDAMLValue( tcNs + "DefProp2" );
			Assertion.AssertNotNull( "Class DefProp0 should not be null", defProp0 );
			Assertion.AssertNotNull( "Class DefProp1 should not be null", defProp1 );
			Assertion.AssertNotNull( "Class DefProp2 should not be null", defProp2 );
			int nP0 = countIteration( defProp0.GetDefinedProperties(), true, "Defined property of DefProp0, closed" );
			int nP0nc = countIteration( defProp0.GetDefinedProperties( false ), true, "Defined property of DefProp0, not closed" );
			int nP1 = countIteration( defProp1.GetDefinedProperties(), true, "Defined property of DefProp1, closed" );
			int nP1nc = countIteration( defProp1.GetDefinedProperties( false ), true, "Defined property of DefProp1, not closed" );
			int nP2 = countIteration( defProp2.GetDefinedProperties(), true, "Defined property of DefProp2, closed" );
			int nP2nc = countIteration( defProp2.GetDefinedProperties( false ), true, "Defined property of DefProp2, not closed" );
			Assertion.AssertEquals( "Defined properties of DefProp0 should number 1", 1, nP0 );
			Assertion.AssertEquals( "Defined properties of DefProp0 (non-closed) should number 1", 1, nP0nc );
			Assertion.AssertEquals( "Defined properties of DefProp1 should number 1", 1, nP1 );
			Assertion.AssertEquals( "Defined properties of DefProp1 (non-closed) should number 0", 0, nP1nc );

			// ijd - this is not working yet: numbers should be 3 and 2 resp.
			//Assertion.AssertEquals( "Defined properties of DefProp2 should number 3", 3, nP2 );
			//Assertion.AssertEquals( "Defined properties of DefProp2 (non-closed) should number 2", 2, nP2nc );
			Assertion.AssertEquals( "Defined properties of DefProp2 should number 3", 2, nP2 );
			Assertion.AssertEquals( "Defined properties of DefProp2 (non-closed) should number 2", 1, nP2nc );

			// Bug report by Thorsten Liebig
			DAMLClass tl_one = (DAMLClass) m0.GetDAMLValue( tcNs + "tl_one" );
			Assertion.AssertNotNull( "Class tl_one should not be null", tl_one );
			int tl_one_supers0 = countIteration( tl_one.Prop_SubClassOf.GetAll( false ), false, null );
			int tl_one_supers1 = countIteration( tl_one.GetSuperClasses( false ), false, null );
			/* TBD: 
			int tl_one_supers2 = countIteration( new StatementSubjectEnumeration( tl_one.listProperties( RDFS.subClassOf ) ), false, null );
			Assertion.AssertEquals( "Should be two super-classes of tl_one by prop_subClassOf", 2, tl_one_supers0 );
			Assertion.AssertEquals( "Should be two super-classes of tl_one by getSuperClasses", 2, tl_one_supers1 );
			Assertion.AssertEquals( "Should be one super-class of tl_one by listProperties", 1, tl_one_supers2 );
			*/

			// Bug report by Andrei S. Lopatenko
			DAMLClass researcher = (DAMLClass) m0.GetDAMLValue( tcNs + "Researcher" );
			Assertion.AssertNotNull( "Class Researcher should not be null", researcher );
			int researcherSupers = countIteration( researcher.GetSuperClasses( false ), true, "Super-class of researcher" );
			Assertion.AssertEquals( "Should be 2 super-classes of researcher", 2, researcherSupers );
		}


		/**
		 * Test equivalance classes
		 */
		public void TestEquivalence()
		{
			string ns = "http://dickinson-i-4/daml/tests/test-cases.daml#";

			// Log.debug( "Starting equivalence tests" );
			DAMLModel m = new DAMLModelImpl();

			// don't allow any additional info to load
			m.GetLoader().SetLoadImportedOntologies( false );
			m.Read( testFiles+"test-cases.daml", "http://dickinson-i-4/daml/tests/test-cases.daml", null );

			// get the root object
			DAMLInstance root = (DAMLInstance) m.GetDAMLValue( ns + "x0" );
			Assertion.AssertNotNull( "Instance x0 should not be null", root );
			Assertion.AssertEquals( "Number of elements in equivalence class should be 4", 4,
				countIteration( root.GetEquivalentValues(), true, "Member of equivalence class to x0: " ) );

			// now it's the classes' turn ...
			DAMLClass cRoot = (DAMLClass) m.GetDAMLValue( ns + "C0" );
			Assertion.AssertNotNull( "Class C0 should not be null", cRoot );
			Assertion.AssertEquals( "Number of elements in equivalence class should be 4", 4,
				countIteration( cRoot.GetSameClasses(), true, "sameClass as C0: " ) );

			// and now the properties ...
			DAMLProperty pRoot = (DAMLProperty) m.GetDAMLValue( ns + "p0" );
			Assertion.AssertNotNull( "Property p0 should not be null", pRoot );
			Assertion.AssertEquals( "Number of elements in equivalence class should be 4", 4,
				countIteration( pRoot.GetSameProperties(), true, "sameProperty as p0: " ) );

			// check that daml:type is recognised as equivalent to rdf:type
			Resource dClass = m.GetResource( ns + "CDaml" );
			Assertion.AssertNotNull( "Resource dClass should not be null", dClass );
			Assertion.Assert( "Resource dClass should be a daml class", dClass is DAMLClass );

			// check that class equivalence is considered during type testing
			DAMLClass cD0 = (DAMLClass) m.GetDAMLValue( ns + "D0" );
			DAMLClass cD1 = (DAMLClass) m.GetDAMLValue( ns + "D1" );
			DAMLInstance d1 = (DAMLInstance) m.GetDAMLValue( ns + "d1" );
			Assertion.AssertNotNull( "Class D0 should not be null", cD0 );
			Assertion.AssertNotNull( "Class D1 should not be null", cD1 );
			Assertion.AssertNotNull( "Instance d1 should not be null", d1 );

			Assertion.Assert( "Instance d1 should have class D1", d1.HasRDFType( cD1 ) );
			Assertion.Assert( "Instance d1 should have class D0", d1.HasRDFType( cD0 ) );

			// check equivalence on properties, pd0 and pd1 are the same
			DAMLProperty pd0 = (DAMLProperty) m.GetDAMLValue( ns + "pd0" );
			DAMLProperty pd1 = (DAMLProperty) m.GetDAMLValue( ns + "pd1" );
			Assertion.AssertNotNull( "Property pd0 should not be null", pd0 );
			Assertion.AssertNotNull( "Property pd1 should not be null", pd1 );

			DAMLInstance d2 = (DAMLInstance) m.GetDAMLValue( ns + "d2" );
			Assertion.AssertNotNull( "Instance d2 should not be null", d2 );

			// check that d2 has d1 as a property value under pd0, and pd1 (it's equivalent)
			Assertion.Assert( "d2 should have d1 as a value for pd0", d2.AccessProperty( pd0 ).HasValue( d1 ) );
			Assertion.Assert( "d2 should have d1 as a value for pd1", d2.AccessProperty( pd1 ).HasValue( d1 ) );

			DAMLProperty pd2 = (DAMLProperty) m.GetDAMLValue( ns + "pd2" );
			DAMLProperty pd3 = (DAMLProperty) m.GetDAMLValue( ns + "pd3" );
			Assertion.AssertNotNull( "Property pd2 should not be null", pd2 );
			Assertion.AssertNotNull( "Property pd3 should not be null", pd3 );

			// we know that 'd1 pd3 d2', which implies 'd1 pd2 d2' since pd2 is a super-prop of pd3
			Assertion.Assert( "d2 should have d1 as a value for pd3", d2.AccessProperty( pd3 ).HasValue( d1 ) );
			Assertion.Assert( "d2 should have d1 as a value for pd2", d2.AccessProperty( pd2 ).HasValue( d1 ) );
		}


		/**
		 * Unit tests on DAMLProperty and its subclasses
		 */
		public void TestProperty()
		{
			// Log.debug( "Starting DAML property tests" );
			DAMLModel m = new DAMLModelImpl();

			m.Read( testFiles+"daml_oil_2001_03/daml+oil-ex.daml", "http://www.daml.org/2001/03/daml+oil-ex", null );
			Assertion.Assert( "loadStatus should be true for successful load", m.GetLoadSuccessful() );
			string ns = "http://www.daml.org/2001/03/daml+oil-ex#";

			// Tests on property objects themselves
			DAMLProperty hasMother = (DAMLProperty) m.GetDAMLValue( ns + "hasMother" );
			Assertion.AssertNotNull( "hasMother property should not be null", hasMother );
			Assertion.Assert( "hasMother property should be a unique property", hasMother.Unique );

			DAMLProperty hasParent = (DAMLProperty) m.GetDAMLValue( ns + "hasParent" );
			Assertion.AssertNotNull( "hasParent property should not be null", hasParent );

			DAMLClass female = (DAMLClass) m.GetDAMLValue( ns + "Female" );
			Assertion.AssertNotNull( "Class Female should not be null", female );
			DAMLClass animal = (DAMLClass) m.GetDAMLValue( ns + "Animal" );
			Assertion.AssertNotNull( "Class Animal should not be null", animal );
			DAMLClass person = (DAMLClass) m.GetDAMLValue( ns + "Person" );
			Assertion.AssertNotNull( "Class Person should not be null", person );

			// range of hasMother includes female
			Assertion.Assert( "Mother should have Female as range", hasMother.Prop_Range.HasValue( female ) );
			Assertion.Assert( "Mother should not have Animal as local domain (prop_domain)", !hasMother.Prop_Domain.HasValue( animal ) );

			bool found = false;
			for (IEnumerator i = hasMother.GetDomainClasses();  !found && i.MoveNext(); ) 
			{
				Object cls = i.Current;
				found = ((DAMLClass) cls).Equals( animal );
			}
			Assertion.Assert( "Mother should have Animal as domain (getDomainClasses)", found );

			// ancestor is transitive
			DAMLObjectProperty hasAncestor = (DAMLObjectProperty) m.GetDAMLValue( ns + "hasAncestor" );
			Assertion.AssertNotNull( "hasAncestor should not be null", hasAncestor );
			Assertion.Assert( "hasAncestor should be transitive", hasAncestor.Transitive );

			// bug report by Michael Sintek: getNext() does not terminate on getAll(false)
			DAMLInstance peter = (DAMLInstance) m.GetDAMLValue( ns + "Peter" );
			Assertion.AssertNotNull( "Instance Peter should not be null", peter );
			DAMLProperty shoesize = (DAMLProperty) m.GetDAMLValue( ns + "shoesize" );
			Assertion.AssertNotNull( "Property shoesize should not be null", shoesize );
			PropertyAccessor paShoesize = peter.AccessProperty( shoesize );
			IEnumerator iShoes = paShoesize.GetAll( false );
			Assertion.AssertEquals( "iShoes iterator should have at least one value", true, iShoes.MoveNext() );
			Object size = iShoes.Current;
			Assertion.AssertNotNull( "size returned from property accessor iterator should not be null", size );
			Assertion.Assert( "size object should be a literal", size is Literal );
			bool nse = false;
			try 
			{
				Object size1 = iShoes.Current;
			}
			catch (Exception) 
			{
				nse = true;
			}
			Assertion.AssertEquals( "Accessing past end of property iterator should throw no such element exception", true, nse );

			// try the same bug test with a multi-valued property
			IEnumerator iSubClassOf = person.Prop_SubClassOf.GetAll( false );
			Assertion.AssertNotNull( "Enumerator over subClassOf values should not be null", iSubClassOf );
			Assertion.Assert( "Iteration of subClassOf should have at least one value", iSubClassOf.MoveNext() );
			int nSupers = countIteration( iSubClassOf, true, "direct super-class of Person = " );
			Assertion.AssertEquals( "Should be 7 direct super-classes of Person", 7, nSupers );

			// another bug report from Michael Sintek - get() on single-valued property does not terminate
			DAMLClass male = (DAMLClass) m.GetDAMLValue( ns + "Male" );
			Assertion.AssertNotNull( "Class Male should not be null", male );
			DAMLCommon femaleDisjoint = (DAMLCommon) female.Prop_DisjointWith.Get();
			Assertion.AssertNotNull( "Value for female.disjointWith should not be null", femaleDisjoint );
			Assertion.Assert( "female.disjointWith should be male", male.Equals( femaleDisjoint ) );

			// another property accessor check
			m.GetLoader().SetLoadImportedOntologies( false );
			m.Read( testFiles+"test-cases.daml", "http://dickinson-i-4/daml/tests/test-cases.daml", null );
			Assertion.Assert( "loadStatus should be true for successful load", m.GetLoadSuccessful() );
			ns = "http://dickinson-i-4/daml/tests/test-cases.daml#";

			DAMLClass subClassCheck3 = (DAMLClass) m.GetDAMLValue( ns + "subClassCheck3" );
			Assertion.AssertNotNull( "Class subClassCheck3 should not be null", subClassCheck3 );

			IEnumerator iSubClassOf3 = subClassCheck3.Prop_SubClassOf.GetAll( false ); // not closed
			Assertion.AssertNotNull( "Enumerator over subClassOf values should not be null", iSubClassOf );
			Assertion.Assert( "Iteration of subClassOf should have at least one value", iSubClassOf3.MoveNext() );
			nSupers = countIteration( iSubClassOf3, true, "property access on subClassCheck3 with closed = false " );
			Assertion.AssertEquals( "Should be 1 non-closed super-classes of subClassCheck3", 1, nSupers );

			iSubClassOf3 = subClassCheck3.Prop_SubClassOf.GetAll( true ); // closed
			Assertion.AssertNotNull( "Enumerator over subClassOf values should not be null", iSubClassOf );
			Assertion.Assert( "Iteration of subClassOf should have at least one value", iSubClassOf3.MoveNext() );
			nSupers = countIteration( iSubClassOf3, true, "property access on subClassCheck3 with closed = true " );
			Assertion.AssertEquals( "Should be 2 closed super-classes of subClassCheck3", 2, nSupers );

			// bug submitted by Michael Sintek: setUseEquivalence(false) does not work with property accessors
			DAMLProperty q = (DAMLProperty) m.GetDAMLValue( ns + "q" );
			Assertion.AssertNotNull( "Property q should not be null", q );
			DAMLInstance qX = (DAMLInstance) m.GetDAMLValue( ns + "qX" );
			Assertion.AssertNotNull( "Instance qX should not be null", qX );

			// with equivalence turned on, there should be four values for q of qX
			int nQ = qX.AccessProperty( q ).Count;
			Assertion.AssertEquals( "There should be 4 values for q of qX (equivalence on)", 4, nQ );

			// turn off equivalence, should only be one value
			m.UseEquivalence = false;
			nQ = qX.AccessProperty( q ).Count;
			Assertion.AssertEquals( "There should be 1 values for q of qX (equivalence off)", 1, nQ );

			// Bug report by Thorsten Liebig
			m.UseEquivalence = true;
			DAMLProperty tlPropTest = (DAMLObjectProperty) m.GetDAMLValue( ns + "TL_PropertyTest" );
			Assertion.AssertNotNull( "Property should not be null", tlPropTest );

			IEnumerator tl_domains = tlPropTest.Prop_Domain.GetAll(false);
			Assertion.AssertEquals( "Property TL_PropertyTest should have a domain of two classes", 2, countIteration( tl_domains, false, null ) );
			IEnumerator tl_ranges = tlPropTest.Prop_Range.GetAll(false);
			Assertion.AssertEquals( "Property TL_PropertyTest should have a range of two classes", 2, countIteration( tl_ranges, false, null ) );

			// bug reported by Wesley Bille
			DAMLClass humanBody = (DAMLClass) m.GetDAMLValue( ns + "HumanBody" );
			Assertion.AssertNotNull( "Class humanBody should not be null", humanBody );
			PropertyAccessor propUnion = humanBody.Prop_UnionOf;
			Assertion.AssertNotNull( "Property accessor should not be null", propUnion );
			Assertion.AssertEquals( "Should be two value in union", 1, propUnion.Count );
			DAMLCommon union = propUnion.GetDAMLValue();
			Assertion.AssertNotNull( "Union should not be null", union );
			Assertion.Assert( "Union value should be a list", union is DAMLList );
			Assertion.AssertEquals( "Should be two values in list", 2, ((DAMLList) union).Count );

		}


		/**
		 * Tests on lists
		 */
		public void TestList()
		{
			// Log.debug( "Starting DAML list tests" );
			DAMLModel m = new DAMLModelImpl();

			m.Read( testFiles+"daml_oil_2001_03/daml+oil-ex.daml", "http://www.daml.org/2001/03/daml+oil-ex", null );
			Assertion.Assert( "loadStatus should be true for successful load", m.GetLoadSuccessful() );
			string ns = "http://www.daml.org/2001/03/daml+oil-ex#";

			// get the Person class
			DAMLClass person = (DAMLClass) m.GetDAMLValue( ns + "Person" );
			Assertion.AssertNotNull( "Person class should not be null", person );
			DAMLClass man = (DAMLClass) m.GetDAMLValue( ns + "Man" );
			Assertion.AssertNotNull( "Man class should not be null", man );
			DAMLClass woman = (DAMLClass) m.GetDAMLValue( ns + "Woman" );
			Assertion.AssertNotNull( "Woman class should not be null", woman );

			// check some basic characteristics of the list
			DAMLList union = (DAMLList) person.Prop_DisjointUnionOf.Get();
			Assertion.AssertNotNull( "union should not be null", union );
			Assertion.AssertEquals( "union should have two values", 2, union.Count );

			// man should be the first element in the list
			DAMLCommon val1 = union.GetFirst();
			Assertion.AssertEquals( "Man should be the first element in the list", man, val1 );

			// woman should be the other element in the list
			DAMLList tail = union.GetRest();
			Assertion.AssertNotNull( "Tail of list should not be null", tail );

			DAMLCommon val2 = tail.GetFirst();
			Assertion.AssertNotNull( "head of tail should not be null", val2 );
			Assertion.AssertEquals( "Woman should be the first element in the tail of the list", woman, val2 );

			DAMLList tail2 = tail.GetRest();
			Assertion.AssertNotNull( "Tail of tail should not be null", tail2 );
			Assertion.Assert( "Remainder of list should be empty", tail2.IsEmpty() );

			// ontologically nonsensical ... just want to test list manipulations
			DAMLClass car = (DAMLClass) m.GetDAMLValue( ns + "Car" );
			Assertion.AssertNotNull( "Class Car should not be null", car );
			union.Add( car );
			// DEBUG dumpModel( m.GetModel() );
			Assertion.AssertEquals( "Union should contain three elements", 3, union.Count );


			// test list deletion - bug submitted by Nicolas Maillot
			DAMLModel nm0 = new DAMLModelImpl();
			DAMLModel nm1 = new DAMLModelImpl();

			DAMLClass nm1A = nm1.CreateDAMLClass("A");
			DAMLClass nm1B = nm1.CreateDAMLClass("B");

			DAMLClass nm0A = nm0.CreateDAMLClass("A");
			DAMLClass nm0B = nm0.CreateDAMLClass("B");
			DAMLClass nm0C = nm0.CreateDAMLClass("C");

			DAMLList nm0L = nm0.CreateDAMLList( null );
			DAMLList nm1L = nm1.CreateDAMLList( null );

			nm0L.Add( nm0A );
			nm0L.Add( nm0B );
			nm0L.Add( nm0C );

			nm1L.Add( nm1A );
			nm1L.Add( nm1B );

			// now remove C from list 0 and check that it is equivalent to list1
			nm0L.Remove( nm0C );
			nm0C.Remove();
			AssertEquals( "Model should be equal to model 1 after removing C", nm0, nm1 );

			// test list deletion 2 - second bug submitted by Nicolas Maillot
			// problems when deleting the first element of the list
			nm0 = new DAMLModelImpl();

			nm0A = nm0.CreateDAMLClass("A");
			nm0B = nm0.CreateDAMLClass("B");
			nm0C = nm0.CreateDAMLClass("C");
			DAMLClass nm0D = nm0.CreateDAMLClass("D");

			nm0L = nm0.CreateDAMLList( null );

			nm0L.Add( nm0D );
			nm0L.Add( nm0A );
			nm0L.Add( nm0B );
			nm0L.Add( nm0C );

			// remove A then C from list
			nm0L.Remove( nm0D );
			nm0L.Remove( nm0C );
			nm0C.Remove();
			nm0D.Remove();
			AssertEquals( "Model should be equal to model 1 after removing C and D", nm0, nm1 );

			// test delete from singleton list
			nm0 = new DAMLModelImpl();

			nm0A = nm0.CreateDAMLClass("A");
			nm0L = nm0.CreateDAMLList( null );

			nm0L.Add( nm0A );
			nm0L.Remove( nm0A );
			nm0A.Remove();

			AssertEquals( "List should have length 0", 0, nm0L.Count );

		}


		/**
		 * Tests on instances
		 */
		public void TestInstance()
		{
			// Log.debug( "Starting DAML instance tests" );
			DAMLModel m = new DAMLModelImpl();

			m.Read( testFiles+"daml_oil_2001_03/daml+oil-ex.daml", "http://www.daml.org/2001/03/daml+oil-ex", null );
			Assertion.Assert( "loadStatus should be true for successful load", m.GetLoadSuccessful() );
			string ns = "http://www.daml.org/2001/03/daml+oil-ex#";

			// count the number of instances loaded from the standard example
			Assertion.AssertEquals( "Number of instances should be 12", 12,
				countIteration( m.ListDAMLInstances(), false, " instance = " ) );

			// test listing the instances of a class
			DAMLClass person = (DAMLClass) m.GetDAMLValue( ns + "Person" );
			Assertion.AssertNotNull( "Person DAML class should not be null", person );
			int nPerson = countIteration( person.GetInstances(), true, "instance of person" );
			Assertion.AssertEquals( "There should be 4 instances of Person in the model", 4, nPerson );
		}


		/**
		 * Tests on DAML datatypes
		 */
		public void TestDatatype()
		{
			// Log.debug( "Starting DAML datatype tests" );
			DAMLModel m = new DAMLModelImpl();

			m.Read( testFiles+"daml_oil_2001_03/daml+oil-ex.daml", "http://www.daml.org/2001/03/daml+oil-ex", null );
			Assertion.Assert( "loadStatus should be true for successful load", m.GetLoadSuccessful() );
			string ns = "http://www.daml.org/2001/03/daml+oil-ex#";

			// get the Person class
			DAMLInstance ian = (DAMLInstance) m.GetDAMLValue( ns + "Ian" );
			Assertion.AssertNotNull( "Instance Ian should not be null", ian );
			DAMLProperty shirtsize = (DAMLProperty) m.GetDAMLValue( ns + "shirtsize" );
			Assertion.AssertNotNull( "Property shirtsize should not be null", shirtsize );
			DAMLProperty shoesize = (DAMLProperty) m.GetDAMLValue( ns + "shoesize" );
			Assertion.AssertNotNull( "Property shoesize should not be null", shoesize );

			DAMLDataInstance sSize = (DAMLDataInstance) ian.GetProperty( shirtsize ).GetObject();
			Assertion.AssertNotNull( "Object ian should have a shirtsize", sSize );
			Object x = sSize.Value;
			Assertion.AssertNotNull( "Value of shirtsize should not be null", x );
			Assertion.AssertEquals( "Shirt size should be a string", typeof(string), x.GetType() );
			Assertion.AssertEquals( "Shirt size should be \"12\"", "12", x );
		}


		/**
		 * Test the removal of DAML objects. We'll load a model, then
		 * delete everything in it one step at a time.
		 */
		public void TestRemove()
		{
			// Log.debug( "Starting DAML remove test" );
			DAMLModel m = new DAMLModelImpl();

			m.Read( testFiles+"daml_oil_2001_03/daml+oil-ex.daml", "http://www.daml.org/2001/03/daml+oil-ex", null );
			Assertion.Assert( "loadStatus should be true for successful load", m.GetLoadSuccessful() );

			// keep going until the model is empty
			bool empty = false;
			while (!empty) 
			{
				// pick an arbitrary DAML resource
				Resource r = null;
				for (StmtEnumerator i = m.ListStatements();  i.MoveNext();  ) 
				{
					Resource r0 = i.Current.GetSubject();

					if (r0 is DAMLCommon) 
					{
						// got one
						r = r0;
						break;
					}
				}

				if (r == null) 
				{
					// all gone
					empty = true;
				}
				else 
				{
					// zap it
					string rURI = r.GetURI();
					bool isClass = r is DAMLClass;

					// Log.debug( "Removing DAML resource " + r );
					((DAMLCommon) r).Remove();

					// shouldn't be a value indexed with this uri now
					// we'll just check the classes to make the code simpler
					if (isClass) 
					{
						for (IEnumerator i = m.ListDAMLClasses();  i.MoveNext(); ) 
						{
							string cURI = ((DAMLClass) i.Current).GetURI();

							if (cURI != null) 
							{
								Assertion.Assert( "DAML class " + rURI + " should have been removed from the index",
									!cURI.Equals( rURI ));
							}
						}
					}

					// shouldn't be in the model either
					int n = 0;
					for (StmtEnumerator j = m.ListStatements( new BBN.RDF.Common.SelectorImpl( r, null, (RDFNode) null ) );  j.MoveNext();  n++);
					Assertion.AssertEquals( "DAML value " + rURI + " should have been removed from the model", 0, n );
				}
			}

			// the model should now be empty
			Assertion.AssertEquals( "Model should now be empty", 0, m.Count );
		}


		/**
		 * Test the creation of new DAML values
		 */
		public void TestCreate()
		{
			DAMLModel m = new DAMLModelImpl();

			string cURI = "http://dickinson-i-4/daml/tests/gen#A";
			DAMLClass c = m.CreateDAMLClass( cURI );
			Assertion.AssertNotNull( "Failed to create new DAML Class " + cURI, c );

			// check that we can find this class again
			bool found = false;
			for (IEnumerator i = m.ListDAMLClasses();  i.MoveNext();   ) 
			{
				if (((DAMLClass) i.Current).Equals( c )) 
				{
					found = true;
				}
			}
			Assertion.Assert( "Could not see class after it was created", found );

			// now create a new instance
			DAMLInstance x = (DAMLInstance) m.CreateDAMLValue( "http://dickinson-i-4/daml/tests/gen#x", c, null );
			Assertion.AssertNotNull( "Failed to create new DAML instance", x );

			found = false;
			for (IEnumerator i = m.ListDAMLInstances();  i.MoveNext();   ) 
			{
				if (((DAMLInstance) i.Current).Equals( x )) 
				{
					found = true;
				}
			}
			Assertion.Assert( "Could not see instance after it was created", found );
		}



		/**
		 * Testing restrictions
		 * @param m
		 */
		public void TestRestriction()
		{
			// Log.debug( "Starting DAML restriction tests" );
			DAMLModel m = new DAMLModelImpl();

			m.Read( testFiles+"daml_oil_2001_03/daml+oil-ex.daml", "http://www.daml.org/2001/03/daml+oil-ex", null );
			Assertion.Assert( "loadStatus should be true for successful load", m.GetLoadSuccessful() );
			string ns = "http://www.daml.org/2001/03/daml+oil-ex#";

			// bug report by Michael Sintek: class cast exception from getting the value of the property iterator
			// get a reference to the Person class
			DAMLClass person = (DAMLClass) m.GetDAMLValue( ns + "Person" );

			// now get a restriction, which we can find as one of the super-classes of Person
			for (IEnumerator i = person.GetSuperClasses();  i.MoveNext(); ) 
			{
				Resource r = (Resource) i.Current;

				if (r is DAMLRestriction) 
				{
					DAMLRestriction restriction = (DAMLRestriction) r;
					PropertyAccessor onPropertyAccessor = restriction.Prop_OnProperty;

					// now get the value from the restriction
					int count = onPropertyAccessor.Count;

					if (count >= 1) 
					{
						Object x = onPropertyAccessor.Get();
						Object y = onPropertyAccessor.GetAll( false ).Current;

						Assertion.AssertNotNull( "Failed to access value of property accessor on restriction", x );
						Assertion.AssertNotNull( "Failed to access value of property accessor on restriction", y );
					}
				}
			}
		}


		/**
		 * Test adding a model to an existing model
		 */
		public void TestModelAdd()
		{
			// Log.debug( "Starting model add test" );
			DAMLModel m = new DAMLModelImpl();

			// create a daml model
			m.Read( testFiles+"test-add-0.daml" );
			Assertion.Assert( "loadStatus should be true for successful load", m.GetLoadSuccessful() );
			//string ns = "http://dickinson-i-4/daml/tests/test-add-0.daml#";

			// create a normal rdf model
			Model m0 = new BBN.RDF.Mem.ModelMem();
			m0.Read( testFiles+"test-add-1.daml" );

			// should be 0 instances in the daml model so far
			Assertion.AssertEquals( "Instance count in DAML model should be 0", 0, countIteration( m.ListDAMLInstances(), true, "instance in test add" ) );

			// now add the RDF data
			m.Add( m0 );

			// now should be 1 instances in the daml model
			Assertion.AssertEquals( "Instance count in DAML model should be 1", 1, countIteration( m.ListDAMLInstances(), true, "instance in test add" ) );
		}


    
		/**
		 * Test reading a model with a syntax error in it.
		 * @throws RDFException
		 */
		/*
			public void testSyntaxError()
				throws RDFException
			{
				Log.debug( "Starting syntax error test" );
				PrintStream oldErr = System.err;
				try {
					ByteArrayOutputStream nOutX = new ByteArrayOutputStream();
					PrintStream nOut = new PrintStream(nOutX);
					System.setOut(nOut);
					DAMLModel m = new DAMLModelImpl();

				// create a daml model
					m.read( "file:modules/rdf/regression/testDAML/test-syntax-error.daml" );
					assertTrue( "loadStatus should be false for unsuccessful load", !m.getLoadSuccessful() );
					assertTrue( "A syntax error should be reported.",nOutX.toString().indexOf("syntax error")!= -1);
				}
				finally {
					System.setOut(oldErr);
				}
			}
		*/

		// Helper Classes
		/////////////////



		/**
		 * Testing equality: case DatatypeProperty
		 */
		/*    public void TestDatatypeProperty()
			{
				eqTest(new EqualityTest("DatatypeProperty") {
						string xml() {
							return "<daml:DatatypeProperty/>";
						}
						void java(DAMLModel m) {
							m.createDAMLDatatypeProperty(null);
						}
				});
			}
		*/
		/**
		 * Testing equality: case ObjectProperty
		 */
		/*    public void TestObjectProperty()
			{
				eqTest(new EqualityTest("ObjectProperty") {
						string xml() {
							return "<daml:ObjectProperty/>";
						}
						void java(DAMLModel m) {
							m.createDAMLObjectProperty(null);
						}
				});
			}
		*/
		/**
		 * Testing equality: case Property
		 */
		/*    public void TestPropertyEq()
			{
				eqTest(new EqualityTest("Property") {
						string xml() {
							return "<daml:Property/>";
						}
						void java(DAMLModel m) {
							m.createDAMLProperty(null);
						}
				});
			}
		*/

		/**
		 * Testing equality: case Datatype
		 */
		/*    public void TestDatatypeEq1()
			{
				eqTest(new EqualityTest("Datatype") {
						string xml() {
							return "<daml:Datatype rdf:about='http://www.w3.org/2000/10/XMLSchema#string'/>";
						}
						void java(DAMLModel m) {
							new DAMLDatatypeImpl("http://www.w3.org/2000/10/XMLSchema#","string",m,null);
						}
				});
			}
		*/
		/**
	 * Testing equality: case Datatype
	 */
		/*    public void TestDatatypeEq2()
			{
				eqTest(new EqualityTest("Datatype") {
						string xml() {
							return "<daml:Datatype rdf:about='http://www.w3.org/2000/10/XMLSchema#string'/>";
						}
						void java(DAMLModel m) {
							m.createDAMLDatatype("http://www.w3.org/2000/10/XMLSchema#string");
						}
				});
			}
		*/
		/*
			public void TestDatatypeRange()
			{
				eqTest(new EqualityTest("Datatype Range") {
						string xml() {
							// Example taken from the DAML+OIL walk-thru
							return
		"<daml:DatatypeProperty rdf:ID='shoesize'>"+
		 " <rdf:type rdf:resource='http://www.daml.org/2001/03/daml+oil#UniqueProperty'/>" +
		 " <daml:range rdf:resource='http://www.w3.org/2000/10/XMLSchema#decimal'/>" +
		"</daml:DatatypeProperty>" +
		"<daml:Property rdf:about='http://www.daml.org/2001/03/daml+oil#range' />" +
		"<daml:Datatype rdf:about='http://www.w3.org/2000/10/XMLSchema#decimal'/>";
						}
						void java(DAMLModel m) {
							DAMLDatatypeProperty shoeSize=m.createDAMLDatatypeProperty("http://example.org/#shoesize");
							shoeSize.SetIsUnique(true);
							shoeSize.Prop_Range.Add(
							m.createDAMLDatatype("http://www.w3.org/2000/10/XMLSchema#decimal") );
						}
				});
			}
		*/


		/**
		 * Concept: the EqualityTest object embodies some java code that adds
		 * stuff to a DAMLModel, and some xml that is the body of an RDF/XML doc.
		 * This test checks that these two different ways of describing a DAML
		 * model are the same.
		 *
		 */
		/*
		private void eqTest(EqualityTest test)
		{

			//        Log.debug( "Starting DAML equality test for " + test.toString() );
			DAMLModel m1 = new DAMLModelImpl();
			test.java(m1);

			Model m2 = new ModelMem();
			Reader rdr = new StringReader(
				"<rdf:RDF " +
				"xmlns:daml='http://www.daml.org/2001/03/daml+oil#' " +
				"xmlns:rdf='http://www.w3.org/1999/02/22-rdf-syntax-ns#'>"
				+ test.xml()
				+ "</rdf:RDF>");
			m2.Read(rdr,"http://example.org/");

			if (! m1.Equals(m2) ) 
			{
				Console.WriteLine("Java:");
				m1.write(new OutputStreamWriter(Console),"RDF/XML-ABBREV");
				Console.WriteLine("XML:");
				m2.write(new OutputStreamWriter(Console),"RDF/XML-ABBREV");
			}

			Assertion.AssertEquals("java code and xml should be equivalent",m1,m2);

		}
		*/


		/**
		 * Dump the model out to a file for debugging
		 */
		/*
		public static void dumpModel( Model m ) 
		{
			dumpModel( m, "model-out.rdf" );
		}
		*/
		/*
		public static void dumpModel( Model m, string fileName ) 
		{
			//        Log.debug( "Dumping model to " + fileName );
			try 
			{
				PrintWriter f = new PrintWriter( new FileWriter( fileName ) );
				m.write( f, "RDF/XML-ABBREV" );
				f.close();
			}
			catch (Exception e) 
			{
				//            Log.severe( "Exception while dumping model: " + e, e );
			}

		}
		*/

		public static void dumpModel( Model m, string filename, bool uri)
		{
			System.IO.TextWriter output = new System.IO.StreamWriter(filename);
			// For each statement...
			StmtEnumerator se = m.ListStatements();
			while(se.MoveNext())
			{
				Statement s = se.Current;
				// print subject uri, predicate uri, object uri
				if (uri) output.WriteLine(s.GetSubject().GetURI()+"  "+s.GetPredicate().GetURI()+"  "+s.GetObject());
				else output.WriteLine(s.GetSubject().GetLocalName()+"    "+s.GetPredicate().GetLocalName()+"    "+s.GetObject());
			}
			output.Close();
		}


		/**
		 * Count the number of things in an iterator, optionally logging them
		 */
		private int countIteration( IEnumerator i, bool doLog, string message ) 
		{
			int count = 0;
			for (;  i.MoveNext();  count++) 
			{
				Object x = i.Current;

				if (doLog) 
				{
					Console.WriteLine( "counting iteration, " + message + x );
				}
			}

			return count;
		}

		private int countIteration( ResEnumerator i, bool doLog, string message ) 
		{
			int count = 0;
			try 
			{
				for (;  i.MoveNext();  count++) 
				{
					Object x = i.Current;

					if (doLog) 
					{
						Console.WriteLine( "counting iteration, " + message + x );
					}
				}
			}
			catch (Exception) 
			{
				//            Log.severe( "RDF exception: " + e, e );
			}

			return count;
		}

		private int countClasses( DAMLModel m ) 
		{
			return countIteration( m.ListDAMLClasses(), false, null );
		}

		private int countProperties( DAMLModel m ) 
		{
			return countIteration( m.ListDAMLProperties(), false, null );
		}

		private int countInstances( DAMLModel m)
		{
			return countIteration( m.ListDAMLInstances(), false, null);
		}



	}
}
