using System;
using BBN.RDF;
using BBN.DAML;
using BBN.RDF.Common;

namespace BBN.DAML.Vocabulary
{

	/// <summary>
	/// Marker interface for a DAML vocabulary.
	/// </summary>
	public interface DAMLVocabulary
	{
		//static const string NAMESPACE_DAML_2001_03_URI = "http://www.daml.org/2001/03/daml+oil#";
		//static const string NAMESPACE_DAML_2000_12_URI = "http://www.daml.org/2000/12/daml+oil#";

		Resource NAMESPACE_DAML{get;}
		DAMLClass List{get;}
		DAMLClass UniqueProperty{get;}
		DAMLClass TransitiveProperty{get;}
		DAMLClass UnambiguousProperty{get;}
		DAMLClass Restriction{get;}
		DAMLClass Ontology{get;}
		DAMLList  Nil{get;}
		DAMLClass Thing{get;}
		DAMLClass Nothing{get;}
		DAMLClass Literal{get;}
		DAMLClass Class{get;}
		DAMLClass Datatype{get;}
		DAMLClass DatatypeProperty{get;}
		DAMLClass ObjectProperty{get;}
		DAMLClass Disjoint{get;}
		DAMLClass Property{get;}

		Property versionInfo{get;}
		Property imports{get;}
		Property disjointWith{get;}
		Property disjointUnionOf{get;}
		Property sameClassAs{get;}
		Property samePropertyAs{get;}
		Property oneOf{get;}
		Property intersectionOf{get;}
		Property unionOf{get;}
		Property complementOf{get;}
		Property equivalentTo{get;}
		Property onProperty{get;}
		Property toClass{get;}
		Property hasValue{get;}
		Property hasClass{get;}
		Property hasClassQ{get;}
		Property cardinality{get;}
		Property minCardinality{get;}
		Property maxCardinality{get;}
		Property cardinalityQ{get;}
		Property minCardinalityQ{get;}
		Property maxCardinalityQ{get;}
		Property inverseOf{get;}
		Property first{get;}
		Property rest{get;}
		Property item{get;}
		Property subPropertyOf{get;}
		Property type{get;}
		Property value{get;}
		Property subClassOf{get;}
		Property domain{get;}
		Property range{get;}
		Property label{get;}
		Property comment{get;}
		Property seeAlso{get;}
		Property isDefinedBy{get;}
		Property sameIndividualAs{get;}
		Property differentIndividualFrom{get;}
	}

	public class DAML_OIL : DAMLVocabulary
	{
		public static readonly string NAMESPACE_DAML_2001_03_URI = "http://www.daml.org/2001/03/daml+oil#";
		public static readonly string NAMESPACE_DAML_2000_12_URI = "http://www.daml.org/2000/12/daml+oil#";
		/// <summary>
		/// The namespace URI used for this vocabulary.
		/// </summary>
		static readonly string URI = NAMESPACE_DAML_2001_03_URI;

		private static DAML_OIL s_instance = new DAML_OIL();

		private static readonly Resource  _namespace_daml;
		public static readonly DAMLClass _List;
		public static readonly DAMLClass _UniqueProperty;
		public static readonly DAMLClass _TransitiveProperty;
		public static readonly DAMLClass _UnambiguousProperty;
		public static readonly DAMLClass _Restriction;
		public static readonly DAMLClass _Ontology;
		public static readonly DAMLList  _Nil;
		public static readonly DAMLClass _Thing;
		public static readonly DAMLClass _Nothing;
		public static readonly DAMLClass _Literal;
		public static readonly DAMLClass _Class;
		public static readonly DAMLClass _Datatype;
		public static readonly DAMLClass _DatatypeProperty;
		public static readonly DAMLClass _ObjectProperty;
		public static readonly DAMLClass _Disjoint;
		public static readonly DAMLClass _Property;

		public static readonly Property _versionInfo;
		public static readonly Property _imports;
		public static readonly Property _disjointWith;
		public static readonly Property _disjointUnionOf;
		public static readonly Property _sameClassAs;
		public static readonly Property _samePropertyAs;
		public static readonly Property _oneOf;
		public static readonly Property _intersectionOf;
		public static readonly Property _unionOf;
		public static readonly Property _complementOf;
		public static readonly Property _equivalentTo;
		public static readonly Property _onProperty;
		public static readonly Property _toClass;
		public static readonly Property _hasValue;
		public static readonly Property _hasClass;
		public static readonly Property _hasClassQ;
		public static readonly Property _cardinality;
		public static readonly Property _minCardinality;
		public static readonly Property _maxCardinality;
		public static readonly Property _cardinalityQ;
		public static readonly Property _minCardinalityQ;
		public static readonly Property _maxCardinalityQ;
		public static readonly Property _inverseOf;
		public static readonly Property _first;
		public static readonly Property _rest;
		public static readonly Property _item;
		public static readonly Property _subPropertyOf;
		public static readonly Property _type;
		public static readonly Property _value;
		public static readonly Property _subClassOf;
		public static readonly Property _domain;
		public static readonly Property _range;
		public static readonly Property _label;
		public static readonly Property _comment;
		public static readonly Property _seeAlso;
		public static readonly Property _isDefinedBy;
		public static readonly Property _sameIndividualAs;
		public static readonly Property _differentIndividualFrom;

		static DAML_OIL()
		{
			_namespace_daml = new ResourceImpl(URI);
			_List = new DAMLClassImpl(URI, "List", null, s_instance);
			_UniqueProperty = new DAMLClassImpl(URI, "UniqueProperty", null, s_instance);
			_TransitiveProperty = new DAMLClassImpl(URI, "TransitiveProperty", null, s_instance);
			_UnambiguousProperty = new DAMLClassImpl(URI, "UnambiguousProperty", null, s_instance);
			_Restriction = new DAMLClassImpl(URI, "Restriction", null, s_instance);
			_Ontology = new DAMLClassImpl(URI, "Ontology", null, s_instance);
			_Nil = new DAMLListImpl(URI,  "Nil", null, s_instance);
			_Thing = new DAMLClassImpl(URI, "Thing", null, s_instance);
			_Nothing = new DAMLClassImpl(URI, "Nothing", null, s_instance);
			_Literal = new DAMLClassImpl(URI, "Literal", null, s_instance);
			_Class = new DAMLClassImpl(URI, "Class", null, s_instance);
			_Datatype = new DAMLClassImpl(URI, "Datatype", null, s_instance);
			_DatatypeProperty = new DAMLClassImpl(URI, "DatatypeProperty", null, s_instance);
			_ObjectProperty = new DAMLClassImpl(URI, "ObjectProperty", null, s_instance);
			_Disjoint = new DAMLClassImpl(URI, "Disjoint", null, s_instance);
			_Property = new DAMLClassImpl(URI, "Property", null, s_instance);

			_versionInfo = new DAMLPropertyImpl(URI, "versionInfo", null, s_instance);
			_imports = new DAMLPropertyImpl(URI, "imports", null, s_instance);
			_disjointWith = new DAMLPropertyImpl(URI, "disjointWith", null, s_instance);
			_disjointUnionOf = new DAMLPropertyImpl(URI, "disjointUnionOf", null, s_instance);
			_sameClassAs = new DAMLPropertyImpl(URI, "sameClassAs", null, s_instance);
			_samePropertyAs = new DAMLPropertyImpl(URI, "samePropertyAs", null, s_instance);
			_equivalentTo = new DAMLPropertyImpl(URI, "equivalentTo", null, s_instance);
			_oneOf = new DAMLPropertyImpl(URI, "oneOf", null, s_instance);
			_intersectionOf = new DAMLPropertyImpl(URI, "intersectionOf", null, s_instance);
			_unionOf = new DAMLPropertyImpl(URI, "unionOf", null, s_instance);
			_complementOf = new DAMLPropertyImpl(URI, "complementOf", null, s_instance);
			_onProperty = new DAMLPropertyImpl(URI, "onProperty", null, s_instance);
			_toClass = new DAMLPropertyImpl(URI, "toClass", null, s_instance);
			_hasValue = new DAMLPropertyImpl(URI, "hasValue", null, s_instance);
			_hasClass = new DAMLPropertyImpl(URI, "hasClass", null, s_instance);
			_hasClassQ = new DAMLPropertyImpl(URI, "hasClassQ", null, s_instance);
			_cardinality = new DAMLPropertyImpl(URI, "cardinality", null, s_instance);
			_cardinalityQ = new DAMLPropertyImpl(URI, "cardinalityQ", null, s_instance);
			_minCardinalityQ = new DAMLPropertyImpl(URI, "minCardinalityQ", null, s_instance);
			_minCardinality = new DAMLPropertyImpl(URI, "minCardinality", null, s_instance);
			_maxCardinality = new DAMLPropertyImpl(URI, "maxCardinality", null, s_instance);
			_maxCardinalityQ = new DAMLPropertyImpl(URI, "maxCardinalityQ", null, s_instance);
			_inverseOf = new DAMLPropertyImpl(URI, "inverseOf", null, s_instance);
			_first = new DAMLPropertyImpl(URI, "first", null, s_instance);
			_rest = new DAMLPropertyImpl(URI, "rest", null, s_instance);
			_item = new DAMLPropertyImpl(URI, "item", null, s_instance);
			_subPropertyOf = new DAMLPropertyImpl(URI, "subPropertyOf", null, s_instance);
			_type = new DAMLPropertyImpl(URI, "type", null, s_instance);
			_value = new DAMLPropertyImpl(URI, "value", null, s_instance);
			_subClassOf = new DAMLPropertyImpl(URI, "subClassOf", null, s_instance);
			_domain = new DAMLPropertyImpl(URI, "domain", null, s_instance);
			_range = new DAMLPropertyImpl(URI, "range", null, s_instance);
			_label = new DAMLPropertyImpl(URI, "label", null, s_instance);
			_comment = new DAMLPropertyImpl(URI, "comment", null, s_instance);
			_seeAlso = new DAMLPropertyImpl(URI, "seeAlso", null, s_instance);
			_isDefinedBy = new DAMLPropertyImpl(URI, "isDefinedBy", null, s_instance);
			_sameIndividualAs = new DAMLPropertyImpl(URI, "sameIndividualAs", null, s_instance);
			_differentIndividualFrom = new DAMLPropertyImpl(URI, "differentIndividualFrom", null, s_instance);
		}// end static constructor

		private DAML_OIL(){}

		public string GetNameSpace()
		{
			return URI;
		}
		public static DAMLVocabulary GetInstance()
		{
			return s_instance;
		}

		public Resource NAMESPACE_DAML{get{return _namespace_daml;}}
		public DAMLClass List{get{return _List;}}
		public DAMLClass UniqueProperty{get{return _UniqueProperty;}}
		public DAMLClass TransitiveProperty{get{return _TransitiveProperty;}}
		public DAMLClass UnambiguousProperty{get{return _UnambiguousProperty;}}
		public DAMLClass Restriction{get{return _Restriction;}}
		public DAMLClass Ontology{get{return _Ontology;}}
		public DAMLList  Nil{get{return _Nil;}}
		public DAMLClass Thing{get{return _Thing;}}
		public DAMLClass Nothing{get{return _Nothing;}}
		public DAMLClass Literal{get{return _Literal;}}
		public DAMLClass Class{get{return _Class;}}
		public DAMLClass Datatype{get{return _Datatype;}}
		public DAMLClass DatatypeProperty{get{return _DatatypeProperty;}}
		public DAMLClass ObjectProperty{get{return _ObjectProperty;}}
		public DAMLClass Disjoint{get{return _Disjoint;}}
		public DAMLClass Property{get{return _Property;}}

		public Property versionInfo{get{return _versionInfo;}}
		public Property imports{get{return _imports;}}
		public Property disjointWith{get{return _disjointWith;}}
		public Property disjointUnionOf{get{return _disjointUnionOf;}}
		public Property sameClassAs{get{return _sameClassAs;}}
		public Property samePropertyAs{get{return _samePropertyAs;}}
		public Property oneOf{get{return _oneOf;}}
		public Property intersectionOf{get{return _intersectionOf;}}
		public Property unionOf{get{return _unionOf;}}
		public Property complementOf{get{return _complementOf;}}
		public Property equivalentTo{get{return _equivalentTo;}}
		public Property onProperty{get{return _onProperty;}}
		public Property toClass{get{return _toClass;}}
		public Property hasValue{get{return _hasValue;}}
		public Property hasClass{get{return _hasClass;}}
		public Property hasClassQ{get{return _hasClassQ;}}
		public Property cardinality{get{return _cardinality;}}
		public Property minCardinality{get{return _minCardinality;}}
		public Property maxCardinality{get{return _maxCardinality;}}
		public Property cardinalityQ{get{return _cardinalityQ;}}
		public Property minCardinalityQ{get{return _minCardinality;}}
		public Property maxCardinalityQ{get{return _maxCardinalityQ;}}
		public Property inverseOf{get{return _inverseOf;}}
		public Property first{get{return _first;}}
		public Property rest{get{return _rest;}}
		public Property item{get{return _item;}}
		public Property subPropertyOf{get{return _subPropertyOf;}}
		public Property type{get{return _type;}}
		public Property value{get{return _value;}}
		public Property subClassOf{get{return _subClassOf;}}
		public Property domain{get{return _domain;}}
		public Property range{get{return _range;}}
		public Property label{get{return _label;}}
		public Property comment{get{return _comment;}}
		public Property seeAlso{get{return _seeAlso;}}
		public Property isDefinedBy{get{return _isDefinedBy;}}
		public Property sameIndividualAs{get{return _sameIndividualAs;}}
		public Property differentIndividualFrom{get{return _differentIndividualFrom;}}
	}

	public class DAML_OIL_2000_12 : DAMLVocabulary
	{
		static readonly string NAMESPACE_DAML_2001_03_URI = "http://www.daml.org/2001/03/daml+oil#";
		static readonly string NAMESPACE_DAML_2000_12_URI = "http://www.daml.org/2000/12/daml+oil#";

		static readonly string URI = NAMESPACE_DAML_2000_12_URI;

		private static DAML_OIL_2000_12 s_instance = new DAML_OIL_2000_12();

		private static readonly Resource  _namespace_daml;
		public static readonly DAMLClass _List;
		public static readonly DAMLClass _UniqueProperty;
		public static readonly DAMLClass _TransitiveProperty;
		public static readonly DAMLClass _UnambiguousProperty;
		public static readonly DAMLClass _Restriction;
		public static readonly DAMLClass _Ontology;
		public static readonly DAMLList  _Nil;
		public static readonly DAMLClass _Thing;
		public static readonly DAMLClass _Nothing;
		public static readonly DAMLClass _Literal;
		public static readonly DAMLClass _Class;
		//public static readonly DAMLClass _Datatype;
		//public static readonly DAMLClass _DatatypeProperty;
		//public static readonly DAMLClass _ObjectProperty;
		public static readonly DAMLClass _Disjoint;
		public static readonly DAMLClass _Property;

		public static readonly Property _versionInfo;
		public static readonly Property _imports;
		public static readonly Property _disjointWith;
		public static readonly Property _disjointUnionOf;
		public static readonly Property _sameClassAs;
		public static readonly Property _samePropertyAs;
		public static readonly Property _oneOf;
		public static readonly Property _intersectionOf;
		public static readonly Property _unionOf;
		public static readonly Property _complementOf;
		public static readonly Property _equivalentTo;
		public static readonly Property _onProperty;
		public static readonly Property _toClass;
		public static readonly Property _hasValue;
		public static readonly Property _hasClass;
		public static readonly Property _hasClassQ;
		public static readonly Property _cardinality;
		public static readonly Property _minCardinality;
		public static readonly Property _maxCardinality;
		public static readonly Property _cardinalityQ;
		public static readonly Property _minCardinalityQ;
		public static readonly Property _maxCardinalityQ;
		public static readonly Property _inverseOf;
		public static readonly Property _first;
		public static readonly Property _rest;
		public static readonly Property _item;
		public static readonly Property _subPropertyOf;
		public static readonly Property _type;
		public static readonly Property _value;
		public static readonly Property _subClassOf;
		public static readonly Property _domain;
		public static readonly Property _range;
		public static readonly Property _label;
		public static readonly Property _comment;
		public static readonly Property _seeAlso;
		public static readonly Property _isDefinedBy;
		public static readonly Property _sameIndividualAs;
		public static readonly Property _differentIndividualFrom;

		static DAML_OIL_2000_12()
		{

			_namespace_daml = new ResourceImpl(URI);
			_List = new DAMLClassImpl(URI, "List", null, s_instance);
			_UniqueProperty = new DAMLClassImpl(URI, "UniqueProperty", null, s_instance);
			_TransitiveProperty = new DAMLClassImpl(URI, "TransitiveProperty", null, s_instance);
			_UnambiguousProperty = new DAMLClassImpl(URI, "UnambiguousProperty", null, s_instance);
			_Restriction = new DAMLClassImpl(URI, "Restriction", null, s_instance);
			_Ontology = new DAMLClassImpl(URI, "Ontology", null, s_instance);
			_Nil = new DAMLListImpl(URI,  "Nil", null, s_instance);
			_Thing = new DAMLClassImpl(URI, "Thing", null, s_instance);
			_Nothing = new DAMLClassImpl(URI, "Nothing", null, s_instance);
			_Literal = new DAMLClassImpl(URI, "Literal", null, s_instance);
			_Class = new DAMLClassImpl(URI, "Class", null, s_instance);
			//_Datatype = new DAMLClassImpl(URI, "Datatype", null, s_instance);
			//_DatatypeProperty = new DAMLClassImpl(URI, "DatatypeProperty", null, s_instance);
			//_ObjectProperty = new DAMLClassImpl(URI, "ObjectProperty", null, s_instance);
			_Disjoint = new DAMLClassImpl(URI, "Disjoint", null, s_instance);
			_Property = new DAMLClassImpl(URI, "Property", null, s_instance);


			_versionInfo = new DAMLPropertyImpl(URI, "versionInfo", null, s_instance);
			_imports = new DAMLPropertyImpl(URI, "imports", null, s_instance);
			_disjointWith = new DAMLPropertyImpl(URI, "disjointWith", null, s_instance);
			_disjointUnionOf = new DAMLPropertyImpl(URI, "disjointUnionOf", null, s_instance);
			_sameClassAs = new DAMLPropertyImpl(URI, "sameClassAs", null, s_instance);
			_samePropertyAs = new DAMLPropertyImpl(URI, "samePropertyAs", null, s_instance);
			_equivalentTo = new DAMLPropertyImpl(URI, "equivalentTo", null, s_instance);
			_oneOf = new DAMLPropertyImpl(URI, "oneOf", null, s_instance);
			_intersectionOf = new DAMLPropertyImpl(URI, "intersectionOf", null, s_instance);
			_unionOf = new DAMLPropertyImpl(URI, "unionOf", null, s_instance);
			_complementOf = new DAMLPropertyImpl(URI, "complementOf", null, s_instance);
			_onProperty = new DAMLPropertyImpl(URI, "onProperty", null, s_instance);
			_toClass = new DAMLPropertyImpl(URI, "toClass", null, s_instance);
			_hasValue = new DAMLPropertyImpl(URI, "hasValue", null, s_instance);
			_hasClass = new DAMLPropertyImpl(URI, "hasClass", null, s_instance);
			_hasClassQ = new DAMLPropertyImpl(URI, "hasClassQ", null, s_instance);
			_cardinality = new DAMLPropertyImpl(URI, "cardinality", null, s_instance);
			_cardinalityQ = new DAMLPropertyImpl(URI, "cardinalityQ", null, s_instance);
			_minCardinalityQ = new DAMLPropertyImpl(URI, "minCardinalityQ", null, s_instance);
			_minCardinality = new DAMLPropertyImpl(URI, "minCardinality", null, s_instance);
			_maxCardinality = new DAMLPropertyImpl(URI, "maxCardinality", null, s_instance);
			_maxCardinalityQ = new DAMLPropertyImpl(URI, "maxCardinalityQ", null, s_instance);
			_inverseOf = new DAMLPropertyImpl(URI, "inverseOf", null, s_instance);
			_first = new DAMLPropertyImpl(URI, "first", null, s_instance);
			_rest = new DAMLPropertyImpl(URI, "rest", null, s_instance);
			_item = new DAMLPropertyImpl(URI, "item", null, s_instance);
			_subPropertyOf = new DAMLPropertyImpl(URI, "subPropertyOf", null, s_instance);
			_type = new DAMLPropertyImpl(URI, "type", null, s_instance);
			_value = new DAMLPropertyImpl(URI, "value", null, s_instance);
			_subClassOf = new DAMLPropertyImpl(URI, "subClassOf", null, s_instance);
			_domain = new DAMLPropertyImpl(URI, "domain", null, s_instance);
			_range = new DAMLPropertyImpl(URI, "range", null, s_instance);
			_label = new DAMLPropertyImpl(URI, "label", null, s_instance);
			_comment = new DAMLPropertyImpl(URI, "comment", null, s_instance);
			_seeAlso = new DAMLPropertyImpl(URI, "seeAlso", null, s_instance);
			_isDefinedBy = new DAMLPropertyImpl(URI, "isDefinedBy", null, s_instance);
			_sameIndividualAs = new DAMLPropertyImpl(URI, "sameIndividualAs", null, s_instance);
			_differentIndividualFrom = new DAMLPropertyImpl(URI, "differentIndividualFrom", null, s_instance);
		}// end static constructor

		private DAML_OIL_2000_12(){}

		public string GetNameSpace()
		{
			return URI;
		}
		public static DAMLVocabulary GetInstance()
		{
			return s_instance;
		}

		public Resource NAMESPACE_DAML{get{return _namespace_daml;}}
		public DAMLClass List{get{return _List;}}
		public DAMLClass UniqueProperty{get{return _UniqueProperty;}}
		public DAMLClass TransitiveProperty{get{return _TransitiveProperty;}}
		public DAMLClass UnambiguousProperty{get{return _UnambiguousProperty;}}
		public DAMLClass Restriction{get{return _Restriction;}}
		public DAMLClass Ontology{get{return _Ontology;}}
		public DAMLList  Nil{get{return _Nil;}}
		public DAMLClass Thing{get{return _Thing;}}
		public DAMLClass Nothing{get{return _Nothing;}}
		public DAMLClass Literal{get{return _Literal;}}
		public DAMLClass Class{get{return _Class;}}
		public DAMLClass Datatype{get{return null;}}         // not in this version
		public DAMLClass DatatypeProperty{get{return null;}} // not in this version
		public DAMLClass ObjectProperty{get{return null;}}   // not in this version
		public DAMLClass Disjoint{get{return _Disjoint;}}
		public DAMLClass Property{get{return _Property;}}

		public Property versionInfo{get{return _versionInfo;}}
		public Property imports{get{return _imports;}}
		public Property disjointWith{get{return _disjointWith;}}
		public Property disjointUnionOf{get{return _disjointUnionOf;}}
		public Property sameClassAs{get{return _sameClassAs;}}
		public Property samePropertyAs{get{return _samePropertyAs;}}
		public Property oneOf{get{return _oneOf;}}
		public Property intersectionOf{get{return _intersectionOf;}}
		public Property unionOf{get{return _unionOf;}}
		public Property complementOf{get{return _complementOf;}}
		public Property equivalentTo{get{return _equivalentTo;}}
		public Property onProperty{get{return _onProperty;}}
		public Property toClass{get{return _toClass;}}
		public Property hasValue{get{return _hasValue;}}
		public Property hasClass{get{return _hasClass;}}
		public Property hasClassQ{get{return _hasClassQ;}}
		public Property cardinality{get{return _cardinality;}}
		public Property minCardinality{get{return _minCardinality;}}
		public Property maxCardinality{get{return _maxCardinality;}}
		public Property cardinalityQ{get{return _cardinalityQ;}}
		public Property minCardinalityQ{get{return _minCardinality;}}
		public Property maxCardinalityQ{get{return _maxCardinalityQ;}}
		public Property inverseOf{get{return _inverseOf;}}
		public Property first{get{return _first;}}
		public Property rest{get{return _rest;}}
		public Property item{get{return _item;}}
		public Property subPropertyOf{get{return _subPropertyOf;}}
		public Property type{get{return _type;}}
		public Property value{get{return _value;}}
		public Property subClassOf{get{return _subClassOf;}}
		public Property domain{get{return _domain;}}
		public Property range{get{return _range;}}
		public Property label{get{return _label;}}
		public Property comment{get{return _comment;}}
		public Property seeAlso{get{return _seeAlso;}}
		public Property isDefinedBy{get{return _isDefinedBy;}}
		public Property sameIndividualAs{get{return _sameIndividualAs;}}
		public Property differentIndividualFrom{get{return _differentIndividualFrom;}}
	}
}
