using System;
using BBN.RDF;

namespace BBN.DAML
{
	/// <summary>
	/// 
	/// </summary>
	public class IntLiteralAccessorImpl : LiteralAccessorImpl, IntLiteralAccessor

	{
		// Constructors
		//////////////////////////////////

		/// <summary>
		/// Construct a new accessor for integer literal values of the given property.
		/// </summary>
		/// <param name="property">The property that this accessor works on</param>
		/// <param name="val">The DAML value that has this property</param>
		public IntLiteralAccessorImpl( Property property, DAMLCommon val ) : base( property, val )
		{
		}



		// External signature methods
		//////////////////////////////////


		/**
		 * Answer the a value of the encapsulated property. If it has no values, answer
		 * null. If it has one value, answer that value. Otherwise, answer an undetermined
		 * member of the set of values.
		 *
		 * @return A value for the encapsulated property in the model, as an integer.
		 * @exception {@link java.lang.RuntimeException} if the property is not defined or cannot be
		 *            interpreted as an integer.
		 */
		public int GetInt() 
		{
			try 
			{
				NodeEnumerator i = GetValues();

				if (i == null  ||  !i.MoveNext()) 
				{
					throw new System.Exception( "No value defined for property " + GetProperty() );
				}
				else 
				{
					return ((Literal) i.Current).GetInteger();
				}
			}
			catch (System.Exception e) 
			{
				Console.Error.WriteLine( "RDF exception when getting literal values: " + e);
				throw new System.Exception( "RDF exception when getting literal values: " + e );
			}
		}


		/**
		 * Add a value to the encapsulated property.
		 *
		 * @param value The value to be added, as an int.
		 */
		public void AddInt( int value ) 
		{
			AddValue( value.ToString() );
		}


		/**
		 * Remove an integer value from the encapsulated property.
		 *
		 * @param value The value to be removed, as an int.
		 */
		public void RemoveInt( int value ) 
		{
			RemoveValue( value.ToString());
		}


		/**
		 * Answer true if the encapsulated property has the given value as one of its
		 * values.
		 *
		 * @param value An int value to test for
		 * @return True if the RDF model contains a statement giving a value for
		 *         the encapsulated property matching the given value.
		 */
		public bool HasIntValue( int value ) 
		{
			return HasValue( value.ToString() );
		}


	}
}
