using System;
using BBN.RDF;

namespace BBN.RDF.Common
{
	/// <summary>
	/// Implementation of an RDF Literal object
	/// </summary>
	public class LiteralImpl : object, Literal
	{
		string literal;
		string language;
		bool wellFormed = false;

		public LiteralImpl(long l)
		{
			literal = l.ToString();
		}
		public LiteralImpl(char l)
		{
			literal = l.ToString();
		}
		public LiteralImpl(float l)
		{
			literal = l.ToString();
		}
		public LiteralImpl(double l)
		{
			literal = l.ToString();
		}
		public LiteralImpl(string s)
		{
			literal = (s==null) ? "" : s;
		}
		public LiteralImpl(string s, string l)
		{
			literal = (s==null) ? "" : s;
			language = l;
		}
		public LiteralImpl(string s, bool wellFormed)
		{
			literal = (s==null) ? "" : s;
			this.wellFormed = wellFormed;
		}
		public LiteralImpl(string s, string l, bool wellFormed)
		{
			literal = (s==null) ? "" : s;
			language = l;
			this.wellFormed = wellFormed;
		}
		public LiteralImpl(object o)
		{
			literal = o.ToString();
		}
		public bool GetBoolean()
		{
			/*if(literal.Equals("true"))
				return true;
			else if(literal.Equals("false"))
				return false;
			else throw new System.Exception("Invalid Boolean Format");*/
			// try the following, it might work:
			return bool.Parse(literal);
		}

		public byte GetByte()
		{
			return byte.Parse(literal);
		}

		public short GetShort()
		{
			return short.Parse(literal);
		}

		public int GetInteger()
		{
			return int.Parse(literal);
		}

		public long GetLong()
		{
			return long.Parse(literal);
		}

		public char GetChar()
		{
			return char.Parse(literal);
		}

		public float GetFloat()
		{
			return float.Parse(literal);
		}

		public double GetDouble()
		{
			return double.Parse(literal);
		}

		public string GetString()
		{
			return literal;
		}

		public object GetObject(ObjectF f)
		{
			try{return f.CreateObject(literal);}
			catch(System.Exception e)
			{
				throw e;
			}
		}

		public string GetLanguage()
		{
			return language;
		}

		public bool IsWellFormed()
		{
			return wellFormed;
		}

		public override bool Equals(object o)
		{
			if (o == null) 
			{
				return false;
			}

			if (o is Literal) 
			{            
				return (literal.Equals(((Literal)o).ToString()) &&
					wellFormed == ((Literal)o).IsWellFormed() &&
					language.Equals(((Literal)o).GetLanguage()));
			} 
			else 
			{
				return false;
			}
		}


		public override string ToString()
		{
			return literal;
		}
		public override int GetHashCode()
		{
			return literal == null ? 0 : literal.GetHashCode();
		}
	}
}
