using System;
using BBN.RDF;

namespace BBN.RDF.Common
{
	/// <summary>
	/// Implementation of the Property interface
	/// </summary>
	/// <author>Troy Self</author>
	public class PropertyImpl : ResourceImpl, Property
	{
		protected int ordinal = 0;

		public PropertyImpl(string uri) : base(uri)
		{
			if(localName == null || localName.Equals(""))
				throw new System.Exception("Invalid Property URI");
			CheckOrdinal();
		}
		public PropertyImpl(string nameSpace, string localName) : base(nameSpace, localName)
		{
			CheckOrdinal();
		}
		public PropertyImpl(string uri, Model m) : base(uri, m)
		{
			if(localName == null || localName.Equals(""))
				throw new System.Exception("Invalid Property URI");
			CheckOrdinal();
		}
		public PropertyImpl(string nameSpace, string localName, Model m)
			: base(nameSpace, localName, m)
		{
			CheckOrdinal();
		}
		public PropertyImpl(string nameSpace, string localName, int ordinal, Model m)
			: base(nameSpace, localName, m)
		{
			this.ordinal = ordinal;
		}
		
		public new string GetNameSpace()
		{
			return nameSpace;
		}
		public int GetOrdinal()
		{
			return ordinal;
		}
		protected void CheckOrdinal()
		{
			char c;

			if(localName[0] == '_' && nameSpace.Equals(Vocabulary.RDF.GetURI())
				&& localName.Length > 1)
			{
				for(int i = 1; i < localName.Length; i++)
				{
					c = localName[i];
					if(c < '0' || c > '9')
						return;
				}
				try{ordinal = int.Parse(localName.Substring(1));}
				catch(System.Exception)
				{
					Console.Error.WriteLine("Ordinal error in " + this.GetType().Name);
				}
			}
		}
		public override Resource Port(Model m)
		{
			if(model == m)
				return this;
			else
			{
				PropertyImpl p = new PropertyImpl(nameSpace, localName, ordinal, m);
				p.id = id;
				return p;
			}
		}
	}
#if DYNAMIC
	public class DynamicPropertyImpl : PropertyImpl
	{
		public DynamicPropertyImpl(string uri, Model model) : base(uri, model)
		{
		}

		public DynamicPropertyImpl(string ns, string localName, Model model) 
			: base(ns, localName, model)
		{
		}

		protected void MaybeLoad()
		{
			if (! ((DynamicModel)GetModel()).DynamicLoad())
				return;

			string uri = GetURI();
			int hash = uri.LastIndexOf('#');
			if (hash != (-1))
				uri = uri.Substring(0, hash);
			DynamicModel model = (DynamicModel) GetModel(); 
			if (! model.IsLoaded(uri))
			{
				model.Read(uri);
			}
		}

		public override StmtEnumerator ListProperties()
		{
			MaybeLoad();
			return base.ListProperties();
		}

		public override StmtEnumerator ListProperties(Property p)
		{
			MaybeLoad();
			return base.ListProperties(p);
		}

		public override Resource Port(Model m)
		{
			if (GetModel() == m)
				return this;
			else return new DynamicPropertyImpl(GetURI(), m);
		}

	}
#endif
}
