// Copyright (C) 2000, 2001 Jason Diamond <http://injektilo.org/>

// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:

// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.

// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

namespace Injektilo.RDF
{
	using System;

	public class RdfStatement
	{
		private RdfSubjectType subjectType;
		private string subject;
		private string predicate;
		private int index;
		private RdfObjectType objectType;
		private string @object;
		private string xmlLang;
		private string id;
		private string source;

		public RdfStatement(
			RdfSubjectType subjectType,
			string subject,
			string predicate,
			int index,
			RdfObjectType objectType,
			string @object,
			string xmlLang,
			string id,
			string source)
		{
			this.subjectType = subjectType;
			this.subject = subject;
			this.predicate = predicate;
			this.index = index;
			this.objectType = objectType;
			this.@object = @object;
			this.xmlLang = xmlLang;
			//			this.id = id;
			if(id == null || id.Length < 1)
				id = this.GetStatementID();
			else
				this.id = id;
			this.source = source;
		}

		public RdfStatement(
			RdfSubjectType subjectType,
			string subject,
			string predicate,
			int index,
			RdfObjectType objectType,
			string @object,
			string xmlLang)
		{
			this.subjectType = subjectType;
			this.subject = subject;
			this.predicate = predicate;
			this.index = index;
			this.objectType = objectType;
			this.@object = @object;
			this.xmlLang = xmlLang;
			this.id = GetStatementID();
			this.source = null;
		}

		public bool SameAs(RdfStatement stmt)
		{
			return (this.Subject.Equals(stmt.Subject) &&
				this.SubjectType.Equals(stmt.SubjectType) &&
				this.ObjectType.Equals(stmt.ObjectType) &&
				this.Object.Equals(stmt.Object) &&
				this.Predicate.Equals(stmt.Predicate));
		}
		public static string HexToBase32(string sHex)
		{
			sHex = sHex.ToUpper();
			string BinaryString = "";
			string BinaryQuad;
			foreach (char ch in sHex)
			{
				if ((byte)ch >= 48 && (byte)ch <= 57)
					BinaryQuad = Convert.ToString(((byte)ch - 48),2);
				else if ((byte)ch >= 65 && (byte)ch <= 90)
					BinaryQuad = Convert.ToString((byte)ch - 55,2);
				else
					continue;
				while (BinaryQuad.Length < 4)
					BinaryQuad = "0" + BinaryQuad;
				BinaryString += BinaryQuad;
			}
			//Pad the front of the binary representation if necessary
			while ((BinaryString.Length % 5) != 0)
				BinaryString = "0" + BinaryString;

			string BinaryQuint;
			int i;
			string Base32 = "";
			while (BinaryString.Length > 0)
			{
				BinaryQuint = BinaryString.Substring(0,5);
				BinaryString = BinaryString.Substring(5,BinaryString.Length - 5);
				i = 0;
				if (BinaryQuint[0] == '1') i += 16;
				if (BinaryQuint[1] == '1') i += 8;
				if (BinaryQuint[2] == '1') i += 4;
				if (BinaryQuint[3] == '1') i += 2;
				if (BinaryQuint[4] == '1') i += 1;
				if (i < 10)
					Base32 += (char)(i + 48);
				else
					Base32 += (char)(i + 55);
			}

			return Base32;
		}
		
		public static string HexToBase64(string sHex)
		{
			//Base 64 strings will be represented as follows:
			//0-9 are...uh, 0-9
			//A-Z are 10-35
			//a-z are 36-61
			//. is 62
			//- is 63
			//These are all valid characters in an XML resource ID.  The only
			//caveat is that a resource ID must begin with a letter, so if
			//that's what the result of this function is being used for...improvise.
			sHex = sHex.ToUpper();
			string sBinary = "";
			string sBinaryQuad;
			foreach (char ch in sHex)
			{
				if ((byte)ch >= 48 && (byte)ch <= 57)
					sBinaryQuad= Convert.ToString(((byte)ch - 48),2);
				else if ((byte)ch >= 65 && (byte)ch <= 90)
					sBinaryQuad = Convert.ToString((byte)ch - 55,2);
				else
					continue;
				while (sBinaryQuad.Length < 4)
					sBinaryQuad = "0" + sBinaryQuad;
				sBinary += sBinaryQuad;
			}
			//Pad the front of the binary representation if necessary
			while ((sBinary.Length % 6) != 0)
				sBinary = "0" + sBinary;

			string sBinarySextuple;
			int i;
			string sBase64 = "";
			while (sBinary.Length > 0)
			{
				sBinarySextuple = sBinary.Substring(0,6);
				sBinary = sBinary.Substring(6,sBinary.Length - 6);
				i = 0;
				if (sBinarySextuple[0] == '1') i += 32;
				if (sBinarySextuple[1] == '1') i += 16;
				if (sBinarySextuple[2] == '1') i += 8;
				if (sBinarySextuple[3] == '1') i += 4;
				if (sBinarySextuple[4] == '1') i += 2;
				if (sBinarySextuple[5] == '1') i += 1;
				if (i < 10)
					sBase64 += (char)(i + 48);
				else if (i < 36)
					sBase64 += (char)(i + 55);
				else if (i < 61)
					sBase64 += (char)(i + 61);
				else if (i == 62)
					sBase64 += '.';
				else    //i == 63
					sBase64 += '-';
			}
			return sBase64;
		}

		private string GetStatementID()
		{
			// concat machine name with time
			Guid g = Guid.NewGuid();
			return "s" + RdfStatement.HexToBase64(g.ToString("N"));
		}
		public override string ToString()
		{
			string result = String.Format("[{0}], [{1}], ", subject, predicate);

			switch(objectType)
			{
				case RdfObjectType.URI:
					result += String.Format("[{0}]", @object);
					break;
				case RdfObjectType.Literal:
					result += String.Format("\"{0}\"", @object);
					break;
				case RdfObjectType.XML:
					result += "XML";
					break;
			}

			return result;
		}

		public RdfSubjectType SubjectType
		{
			get
			{
				return subjectType;
			}
		}

		public string Subject
		{
			get
			{
				return subject;
			}
			set
			{
				subject = value;
			}
		}

		public string Predicate
		{
			get
			{
				return predicate;
			}
		}

		public int Index
		{
			get
			{
				return index;
			}
		}

		public RdfObjectType ObjectType
		{
			get
			{
				return objectType;
			}
		}

		public string Object
		{
			get
			{
				return @object;
			}
			set
			{
				@object = value;
			}
		}

		public string XmlLang
		{
			get
			{
				return xmlLang;
			}
		}
		
		public string Id
		{
			get
			{
				return id;
			}
		}

		public string Source
		{ 
			get
			{ 
				return source;
			}
		}

		public string PredicateNameSpace
		{
			get
			{
				int hash = predicate.LastIndexOf('#');
				return predicate.Substring(0, hash+1);
			}
		}
		public string PredicateLocalName
		{
			get
			{
				int hash = predicate.LastIndexOf('#');
				return predicate.Substring(hash+1);
			}
		}
	}
}
