using System;
using BBN.RDF;

namespace BBN.RDF.Common
{
	/// <summary>
	/// Summary description for SelectorImpl.
	/// </summary>
	public class SelectorImpl : Selector
	{
		protected Resource subject;
		protected Property predicate;
		protected RDFNode rObject;

		public SelectorImpl() : this(null, null, (RDFNode)null)
		{}
		public SelectorImpl(Resource subject, Property predicate, RDFNode o)
		{
			this.subject = subject;
			this.predicate = predicate;
			rObject = o;
		}
		public SelectorImpl(Resource subject, Property predicate, bool o)
			: this(subject, predicate, new LiteralImpl(o))
		{}
		public SelectorImpl(Resource subject, Property predicate, char o)
			: this(subject, predicate, new LiteralImpl(o))
		{}
		public SelectorImpl(Resource subject, Property predicate, long o)
			: this(subject, predicate, new LiteralImpl(o))
		{}
		public SelectorImpl(Resource subject, Property predicate, float o)
			: this(subject, predicate, new LiteralImpl(o))
		{}
		public SelectorImpl(Resource subject, Property predicate, double o)
			: this(subject, predicate, new LiteralImpl(o))
		{}
		public SelectorImpl(Resource subject, Property predicate, string o)
		{
			this.subject = subject;
			this.predicate = predicate;
			if(o != null)
				rObject = new LiteralImpl(o);
			else rObject = null;
		}
		public SelectorImpl(Resource subject, Property predicate, string o, string l)
		{
			this.subject = subject;
			this.predicate = predicate;
			if(o != null)
				rObject = new LiteralImpl(o, l);
			else rObject = null;
		}
		public SelectorImpl(Resource subject, Property predicate, object o)
		{
			this.subject = subject;
			this.predicate = predicate;
			if(o != null)
				rObject = new LiteralImpl(o);
			else rObject = null;
		}
		public Resource GetSubject()
		{
			return subject;
		}
		public Property GetPredicate()
		{
			return predicate;
		}
		public RDFNode GetObject()
		{
			return rObject;
		}
		public bool Test(Statement s)
		{
			return (subject == null || subject.Equals(s.GetSubject()))
				&& (predicate == null || predicate.Equals(s.GetPredicate()))
				&& (rObject == null || rObject.Equals(s.GetObject()))
				&& Selects(s);
		}
		public bool Selects(Statement s)
		{
			return true;
		}
	}
}
