using System;

namespace BBN.RDF.Common
{

	public interface Source
	{

		/// <summary>
		/// Compare two source objects based on the uri.
		/// </summary>
		/// <param name="ob">The object to compare</param>
		/// <returns>true if the sources are equal</returns>
		bool Equals(Object ob);

		/// <summary>
		/// The uri of this source.
		/// </summary>
		string URI 
		{
			get;
		}
    
	}

	/// <summary>
	/// Implementation of a Source
	/// </summary>
	public class SourceImpl : Source
	{
		/// <summary>
		/// The uri of this source.
		/// </summary>
		protected string uri;

		/// <summary>
		/// Create a new source.
		/// </summary>
		/// <param name="uri">The uri of the source.</param>
		public SourceImpl(string uri) : base()
		{
			this.uri = uri;
		}


		public string URI 
		{
			get 
			{
				return uri;
			}
		}

		public override bool Equals(Object ob)
		{
			if ((ob is SourceImpl) &&
				(uri.Equals(((SourceImpl)ob).URI)))
			{
				return true;	
			} 
			else 
			{
				return false;
			}
		}

		public override int GetHashCode()
		{
			return uri.GetHashCode();
		}


		public override string ToString()
		{
			return uri;
		}
	

	}
	
}
