using System;
using BBN.RDF;
using BBN.RDF.Common;
using BBN.DAML.Vocabulary;

namespace BBN.DAML
{
	/// <summary>
	/// 
	/// </summary>
	public class VocabularyManager
	{

    // External signature methods
    //////////////////////////////////

    /**
     * Answer the vocabulary that corresponds to the namespace of the
     * given resource. By default, answer the most recent vocabulary.
     *
     * @param resource The RDF resource denoting a namespace
     * @return a DAML vocabulary object for the namespace
     */
    public static DAMLVocabulary GetVocabulary( Resource resource ) {
        return GetVocabulary( resource.GetURI() );
    }


    /**
     * Answer the vocabulary that corresponds to the namespace of the
     * given URI. By default, answer the most recent vocabulary.
     *
     * @param uri A URI denoting a namespace
     * @return a DAML vocabulary object for the namespace
     */
    public static DAMLVocabulary GetVocabulary( string uri ) {
        if (uri != null) {
            // pull out the namespace of the uri
            int splitPoint = Util.SplitNamespace( uri );
            string ns = (splitPoint < 0) ? uri : uri.Substring( 0, splitPoint );

            // test the known namespaces
            if (ns != null) {
                if (ns.Equals( DAML_OIL.NAMESPACE_DAML_2000_12_URI)) {
                    return DAML_OIL_2000_12.GetInstance();
                }

                // add further namespace tests here as the namespaces are defined.
            }
        }

        // to get here, we assume the default vocabulary
        return GetDefaultVocabulary();
    }


    /**
     * Answer the default (latest) vocabulary.
     *
     * @return a DAML+OIL vocabulary
     */
    public static DAMLVocabulary GetDefaultVocabulary() {
        return DAML_OIL.GetInstance();
    }
	}
}
